/*#######################################################################################
VERSION 1.2
Einfache 3D Graphik-Lib fr die Xilinx XC9572 Grafikkarte von Ulrich Radig

Es ist ein unoptimiertes Rohgerst, das einfach eine 3D affine Transformation umsetzt,
diese dann auf ein 2D Koordinatensystem projeziert.

2D und 3D VERSION 1.1 Graphik-Lib Copyright (C) 2006 by SAAM KARL
www.elektronik.karlsaam.de

Modify to run on the rCos-Backend/STM32 by Oliver Dippel <o.dippel@gmx.de>

#######################################################################################*/

#include <rc.h>
#include <draw2d.h>

#ifdef SUPPORT_3D

#define M_PI		3.14159265
#define deg             (M_PI / 180)

int OX, OY;
int wx, wy, wz;
double sx, sy, sz;
double t;

double cos_wx;
double sin_wx;
double cos_wy;
double sin_wy;
double cos_wz;
double sin_wz;

void Draw3D_Init (int Xa, int Ya, int Xb, int Yb) {
	OX = (Xb + Xa) / 2;
	OY = (Yb - Ya) / 2;
	Draw3D_Transform_Set(0, 0, 0, 1.0, 1.0, 1.0, 800.0);
}

void Draw3D_Transform_Set (int w1, int w2, int w3, double s1, double s2, double s3, double t1) {
	wx = w1;
	wy = w2;
	wz = w3;
	sx = s1;
	sy = s2;
	sz = s3;
	t = t1;
	cos_wx = cos(wx * deg);
	sin_wx = sin(wx * deg);
	cos_wy = cos(wy * deg);
	sin_wy = sin(wy * deg);
	cos_wz = cos(wz * deg);
	sin_wz = sin(wz * deg);
}

void Draw3D_Line (int x1, int y1, int z1, int x2, int y2, int z2) {
	double xx = x1 * sx;
	double yy = y1 * sy;
	double zz = z1 * sz;
	double xxx = xx * cos_wz + yy * sin_wz;
	double yyy = yy * cos_wz - xx * sin_wz;
	double zzz = zz;
	xx = xxx * cos_wy - zzz * sin_wy;
	yy = yyy;
	zz = xxx * sin_wy + zzz * cos_wy;
	xxx = xx;
	xxx = yy * cos_wx + zz * sin_wx;
	yyy = zz * cos_wx - yy * sin_wx;
	long xa = floor(xxx / (zzz + t) * t + 0.5) + OX;
	long ya = OY - floor(yyy / (zzz + t) * t + 0.5);
	xx = x2 * sx;
	yy = y2 * sy;
	zz = z2 * sz;
	xxx = xx * cos_wz + yy * sin_wz;
	yyy = yy * cos_wz - xx * sin_wz;
	zzz = zz;
	xx = xxx * cos_wy - zzz * sin_wy;
	yy = yyy;
	zz = xxx * sin_wy + zzz * cos_wy;
	xxx = xx;
	xxx = yy * cos_wx + zz * sin_wx;
	yyy = zz * cos_wx - yy * sin_wx;
	long xb = floor(xxx / (zzz + t) * t + 0.5) + OX;
	long yb = OY - floor(yyy / (zzz + t) * t + 0.5);
	Draw2D_Line(xa, ya, xb, yb, LCD_FG);
}

void Draw3D_Cube (int mx, int my, int mz, int SizeX, int SizeY, int SizeZ) {
	int hx = SizeX / 2;
	int hy = SizeY / 2;
	int hz = SizeZ / 2;
	Draw3D_Line(mx - hx, my - hy, mz - hz, mx + hx, my - hy, mz - hz);
	Draw3D_Line(mx + hx, my - hy, mz - hz, mx + hx, my + hy, mz - hz);
	Draw3D_Line(mx + hx, my + hy, mz - hz, mx - hx, my + hy, mz - hz);
	Draw3D_Line(mx - hx, my + hy, mz - hz, mx - hx, my - hy, mz - hz);
	Draw3D_Line(mx - hx, my - hy, mz + hz, mx + hx, my - hy, mz + hz);
	Draw3D_Line(mx + hx, my - hy, mz + hz, mx + hx, my + hy, mz + hz);
	Draw3D_Line(mx + hx, my + hy, mz + hz, mx - hx, my + hy, mz + hz);
	Draw3D_Line(mx - hx, my + hy, mz + hz, mx - hx, my - hy, mz + hz);
	Draw3D_Line(mx - hx, my - hy, mz - hz, mx - hx, my - hy, mz + hz);
	Draw3D_Line(mx + hx, my - hy, mz - hz, mx + hx, my - hy, mz + hz);
	Draw3D_Line(mx + hx, my + hy, mz - hz, mx + hx, my + hy, mz + hz);
	Draw3D_Line(mx - hx, my + hy, mz - hz, mx - hx, my + hy, mz + hz);
}

#endif

