/*
Copyright (C) 2008-2009  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include "rc.h"
#include "rc_adc.h"

#define ADC_NUM_CONVERTS	3
#define ADC_NUM_BUFFER		10

#ifdef PLATFORM_STM32
#define ADC_SPEED ADC_SampleTime_239Cycles5
uint8_t ADC_CHANNEL_NUMS[] = ADC_INPUT_LIST;
#endif


#ifdef PLATFORM_SDL
extern uint16_t joy_value[20];
#endif

uint16_t GetADCChanel (uint8_t channel) {
#ifdef PLATFORM_STM32
	uint32_t val = 0;
	uint8_t n = 0;
	ADC_RegularChannelConfig(ADC1, ADC_CHANNEL_NUMS[channel], 1, ADC_SampleTime_13Cycles5);
	for (n = 0; n < 10; n++) {
		ADC_SoftwareStartConvCmd(ADC1, ENABLE);
		while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
		val += ADC_GetConversionValue(ADC1);
	}
	return (uint16_t)(val / 10);
#endif
#ifdef PLATFORM_SDL
	return (uint16_t)(joy_value[channel]);
#endif
}

int16_t Rc_AdcRead (int8_t id) {
	int32_t value = 0;
#ifdef PLATFORM_STM32
	if (id < 4) {
		value = (GetADCChanel(id) - 2048) + Rc_Calibrate_center[id];
		if (value > Rc_Calibrate_max[id]) {
			Rc_Calibrate_max[id] = value;
		}
		if (value < Rc_Calibrate_min[id]) {
			Rc_Calibrate_min[id] = value;
		}
		if (value > 0) {
			value = value * 1024 / Rc_Calibrate_max[id];
		} else if (value < 0) {
			value = value * 1024 / Rc_Calibrate_min[id] * -1;
		}
	} else if (id < ADC_CHANNELS_MAX) {
		value = (GetADCChanel(id) - 2048) / 2;
	}
	return value;
#endif
#ifdef PLATFORM_SDL
	return (GetADCChanel(id) - 2048) / 2;
#endif
}

void Rc_AdcInit (void) {
#ifdef PLATFORM_STM32
	GPIO_InitTypeDef GPIO_InitStructure;

        GPIO_InitStructure.GPIO_Pin = ADC_INPUT_PINS_A;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
        GPIO_Init(GPIOA, &GPIO_InitStructure);

        GPIO_InitStructure.GPIO_Pin = ADC_INPUT_PINS_B;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
        GPIO_Init(GPIOB, &GPIO_InitStructure);

        GPIO_InitStructure.GPIO_Pin = ADC_INPUT_PINS_C;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
        GPIO_Init(GPIOC, &GPIO_InitStructure);

	ADC_InitTypeDef ADC_InitStructure;
	ADC_StructInit(&ADC_InitStructure);
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = 1;
	ADC_Init(ADC1, &ADC_InitStructure);
	ADC_Cmd(ADC1, ENABLE);
#endif
}

