/*
Copyright (C) 2009  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include <string.h>
#include <rc.h>
#include <usart.h>
#include <sd.h>
#include <rc_control.h>

#ifdef SUPPORT_USART

volatile uint8_t TxBuffer[USART_MAX][128];
volatile uint8_t RxBuffer[USART_MAX][128];
volatile uint8_t Tx_Flag[USART_MAX];
volatile uint16_t TxCounter[USART_MAX];
volatile uint16_t NbrOfDataToTransfer[USART_MAX];


void USART_SEND (uint8_t device_id, char *data, uint16_t count) {
	memcpy(&TxBuffer[device_id], data, count);
	NbrOfDataToTransfer[device_id] = count;
	TxCounter[device_id] = 0;
}

void USART_SEND_STR (uint8_t device_id, char *data) {
	uint16_t count = strlen(data);
	memcpy(&TxBuffer[device_id], data, count);
	NbrOfDataToTransfer[device_id] = count;
	TxCounter[device_id] = 0;
}

void USART_TX_START (uint8_t device_id) {
	TxCounter[device_id] = 0;
	USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
	switch (device_id) {
		case 0:
			USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
		break;
		case 1:
			USART_ITConfig(USART2, USART_IT_TXE, ENABLE);
		break;
		case 2:
			USART_ITConfig(USART3, USART_IT_TXE, ENABLE);
		break;
	}
}

void USART_SEND_OK (uint8_t device_id) {
	USART_SEND(device_id, "OK\n", 3);
}

void USART1_IRQHandler (void) {
	static uint8_t data_csum = 0;
	static uint8_t cmd_mode = 0;
	static uint16_t cmd_counter = 0;
	uint8_t csum;
	uint8_t RX;
	if(USART_GetITStatus(USART1, USART_IT_TXE) != RESET) {
		USART_SendData(USART1, TxBuffer[0][TxCounter[0]++]);
		if(TxCounter[0] == NbrOfDataToTransfer[0]) {
			USART_ITConfig(USART1, USART_IT_TXE, DISABLE);
			NbrOfDataToTransfer[0] = 0;
			TxCounter[0] = 0;
		}    
	}
	if(USART_GetITStatus(USART1, USART_IT_RXNE) != RESET) {
		RX = USART_ReceiveData(USART1);
		switch (cmd_mode) {
			case FE_CMD_NONE:
				if (RX < FE_CMD_LAST) {
					cmd_mode = RX;
					data_csum = 255 ^ cmd_mode;
					cmd_counter = 0;
				} else {
					Tx_Flag[0] = 254;
					cmd_mode = FE_CMD_NONE;
					cmd_counter = 0;
				}
			break;
			case FE_CMD_SERVO_TEST:
				if (cmd_counter < 1) {
					RxBuffer[0][cmd_counter++] = RX;
					data_csum ^= RX;
				} else {
					csum = RX;
					if (csum == data_csum) {
						Servo_Testflag = RxBuffer[0][0] + 1;
						Tx_Flag[0] = 253;
					} else {
						Tx_Flag[0] = 255;
					}
					cmd_mode = FE_CMD_NONE;
				}
			break;
			case FE_CMD_CONTROL_VALUE_SET:
			case FE_CMD_CONTROL_SET:
			case FE_CMD_PPMENCODER1_MENCODER_SET:
			case FE_CMD_PPMENCODER1_SET:
			case FE_CMD_PPMENCODER2_SET:
			case FE_CMD_OUTPUT_SET:
			case FE_CMD_EXPO_SET:
			case FE_CMD_LIMITER_SET:
			case FE_CMD_ATV_SET:
			case FE_CMD_DUALRATE_SET:
			case FE_CMD_MOTION_SET:
			case FE_CMD_MIXER_SET:
				if (cmd_counter < FeCmdLen[cmd_mode]) {
					RxBuffer[0][cmd_counter++] = RX;
					data_csum ^= RX;
				} else {
					csum = RX;
					if (csum == data_csum) {
						Tx_Flag[0] = cmd_mode;
					} else {
						Tx_Flag[0] = 255;
					}
					cmd_mode = FE_CMD_NONE;
				}
			break;
			default:
				csum = RX;
				if (csum == data_csum) {
					Tx_Flag[0] = cmd_mode;
				} else {
					Tx_Flag[0] = 255;
				}
				cmd_mode = FE_CMD_NONE;
			break;
		}
	}
}

void USART2_IRQHandler (void) {
	uint8_t RX;
	if(USART_GetITStatus(USART2, USART_IT_TXE) != RESET) {
		USART_SendData(USART2, TxBuffer[1][TxCounter[1]++]);
		if(TxCounter[1] == NbrOfDataToTransfer[1]) {
			USART_ITConfig(USART2, USART_IT_TXE, DISABLE);
			NbrOfDataToTransfer[1] = 0;
			TxCounter[1] = 0;
		}    
	}
	if(USART_GetITStatus(USART2, USART_IT_RXNE) != RESET) {
		RX = USART_ReceiveData(USART2);
	}
}

void USART3_IRQHandler (void) {
	int16_t value = 0;
	static uint8_t cmd_mode = 0;
	static uint16_t cmd_counter = 0;
	static uint8_t cmd_buffer[200];
	static uint8_t last = 0;
	uint8_t RX;
	if(USART_GetITStatus(USART3, USART_IT_RXNE) != RESET) {
		RX = USART_ReceiveData(USART3);
		if (cmd_mode == 1) {
			if (RX == ACT_DSL_GET_FREQTABLE_REP) {
				cmd_mode = ACT_DSL_GET_FREQTABLE;
				cmd_counter = 0;
			} else if (RX == ACT_DSL_GET_INFO_REP) {
				cmd_mode = ACT_DSL_GET_INFO;
				cmd_counter = 0;
			} else if (RX == 31) {
				cmd_mode = 31;
				cmd_counter = 0;
				if (Rc_LS_Calibrate_flag == 1) {
					Rc_LS_Calibrate_flag = 2;
					Rc_LS_Calibrate_center[0] = 0;
					Rc_LS_Calibrate_center[1] = 0;
					Rc_LS_Calibrate_center[2] = 0;
					Rc_LS_Calibrate_center[3] = 0;
				} else if (Rc_LS_Calibrate_flag == 2) {
					Rc_LS_Calibrate_flag = 0;
					Rc_LS_Calibrate_center[0] = ControlRead(50) * -1;
					Rc_LS_Calibrate_center[1] = ControlRead(51) * -1;
					Rc_LS_Calibrate_center[2] = ControlRead(52) * -1;
					Rc_LS_Calibrate_center[3] = ControlRead(53) * -1;
				}
			} else {
				cmd_mode = 0;
			}

		} else if (cmd_mode == 31) {
			if (cmd_counter == 8) {
				value = ((last<<8) + RX - 10000) / 32 + Rc_LS_Calibrate_center[0];
				if (value < 126 && value > -126) {
					ControlWrite(50, value);
				}
				cmd_counter++;
			} else if (cmd_counter == 10) {
				value = ((last<<8) + RX - 10000) / 32 + Rc_LS_Calibrate_center[1];
				if (value < 126 && value > -126) {
					ControlWrite(51, value);
				}
				cmd_counter++;
			} else if (cmd_counter == 14) {
				value = ((last<<8) + RX - 10000) / 32 + Rc_LS_Calibrate_center[2];
				if (value < 126 && value > -126) {
					ControlWrite(52, value);
				}
				cmd_counter++;
			} else if (cmd_counter == 16) {
				value = ((last<<8) + RX - 10000) / 32 + Rc_LS_Calibrate_center[3];
				if (value < 126 && value > -126) {
					ControlWrite(53, value);
				}
				cmd_counter++;
//			} else if (cmd_counter == 18) {
//				ControlWrite(55, ((last<<8) + RX - 10000) / 60);
//				cmd_counter++;
//			} else if (cmd_counter == 20) {
//				ControlWrite(56, ((last<<8) + RX - 10000) / 60);
//				cmd_counter++;
//			} else if (cmd_counter == 22) {
//				ControlWrite(57, ((last<<8) + RX - 10000) / 60);
//				cmd_counter++;
//			} else if (cmd_counter == 24) {
//				ControlWrite(58, ((last<<8) + RX - 10000) / 60);
//				cmd_counter++;
			} else if (cmd_counter < 26) {
				cmd_counter++;
			} else {
				cmd_mode = 0;
			}

		} else if (cmd_mode == ACT_DSL_GET_INFO) {
			if (cmd_counter == 0) {
				cmd_counter++;
			} else if (cmd_counter < 26) {
				cmd_buffer[cmd_counter++ - 1] = RX;
			} else {
				USART_SEND(0, (char *)cmd_buffer, 26);
				cmd_mode = 0;
			}
		} else if (cmd_mode == ACT_DSL_GET_FREQTABLE) {
			if (cmd_counter == 0) {
				cmd_counter++;
			} else if (cmd_counter < 48) {
				cmd_buffer[cmd_counter++ - 1] = RX;
			} else {
				USART_SEND(0, (char *)cmd_buffer, 48);
				cmd_mode = 0;
			}
		} else {
			if (RX == 255 && last == 255) {
				cmd_mode = 1;
			}
		}
		last = RX;
	}
	if(USART_GetITStatus(USART3, USART_IT_TXE) != RESET) {   
		USART_SendData(USART3, TxBuffer[2][TxCounter[2]++]);
		if(TxCounter[2] == NbrOfDataToTransfer[2]) {
			USART_ITConfig(USART3, USART_IT_TXE, DISABLE);
			NbrOfDataToTransfer[2] = 0;
			TxCounter[2] = 0;
		}    
	}
}


#endif
