/*
Copyright (C) 2008  Oliver Dippel <o.dippel@gmx.de>

The Init-Sequences are get from: http://www.mikrocontroller.net/topic/31403

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include <stdint.h>
#include <string.h>
#include <rc.h>
#include <rc_adc.h>
#include <draw2d.h>
#include <spi.h>
#include <s65.h>

#ifdef SUPPORT_LCD
#ifdef SUPPORT_S65

static void S65_Send (uint8_t data, uint8_t cd) {
	Draw2D_SEL_LO;
	if (cd == Draw2D_CMD) {
		Draw2D_DS_HI;
	} else {
		Draw2D_DS_LO;
	}
	SPI_Write_Byte(data);
	Draw2D_SEL_HI;
}

static void S65_Send_CMD_Word (uint16_t dat) {
	S65_Send((dat>>8), Draw2D_CMD);
	S65_Send(dat, Draw2D_CMD);
}

void S65_Update (void) {
	uint16_t n;
	uint16_t n2;
	uint16_t n3;
	uint8_t h2 = Draw2D_H / 8;
	SPI_Fast();
	S65_Send_CMD_Word(0x0700); // X = 0
	S65_Send_CMD_Word(0x0600); // Y = 0
	Draw2D_DS_LO;
	Draw2D_SEL_LO;
	for (n = 0; n < h2; n++) {
		for (n2 = 0; n2 < 8; n2++) {
			for (n3 = 0; n3 < Draw2D_W; n3++) {
#ifdef S65_LANDSCAPE
				if (Framebuffer[n * Draw2D_W + (Draw2D_W - n3 - 1)] & (1<<n2)) {
#else
				if (Framebuffer[n * Draw2D_W + n3] & (1<<n2)) {
#endif
					if (n < 1) {
#ifdef SUPPORT_HWSPI
						SPI_I2S_SendData(HWSPI, 0x07);
#else
						SPI_Write_Byte(0x07);
#endif
					} else {
#ifdef SUPPORT_HWSPI
						SPI_I2S_SendData(HWSPI, Rc_Model_forecolor);
#else
						SPI_Write_Byte(Rc_Model_forecolor);
#endif
					}
				} else {
					if (n < 1) {
#ifdef SUPPORT_HWSPI
						SPI_I2S_SendData(HWSPI, 0xFD);
#else
						SPI_Write_Byte(0xFD);
#endif
					} else {
#ifdef SUPPORT_HWSPI
						//SPI_Write_Byte(logo[(y * 66) + x], Draw2D_DATA);
						SPI_Write_Byte(Rc_Model_backcolor);
#else
						SPI_I2S_SendData(HWSPI, Rc_Model_backcolor);
#endif
					}
				}
			}
		}
	}
	Draw2D_SEL_HI;
}

static void S65_Clear (void) {
	uint16_t i;
	S65_Send_CMD_Word(0x0600);
	S65_Send_CMD_Word(0x0700);
	for (i = 0; i < Draw2D_W * Draw2D_H; i++){
		S65_Send((dat>>8), Draw2D_DATA);
		S65_Send(dat, Draw2D_DATA);
	}
}

void S65_Init (void) {
	unsigned char i;
	Draw2D_RST_LO;
	Draw2D_SEL_HI;
	SPI_CLK_LO();
	Draw2D_DS_HI;
	delay_ms(1);
	Draw2D_RST_HI;
	SPI_Fast();
	for(i = 0; i < 24; i++) {
		S65_Send(S65_INIT1[i], Draw2D_CMD);
	}
	delay_ms(7);
	for(i = 0; i < 40; i++) {
		S65_Send(S65_INIT2[i], Draw2D_CMD);
	}
	delay_ms(50);
	for(i = 0; i < 2; i++) {
		S65_Send(S65_INIT3[i], Draw2D_CMD);
	}
	delay_ms(50);
	S65_Send_CMD_Word(0xEF90);
#ifdef S65_LANDSCAPE
	S65_Send_CMD_Word(0x0504); // Rotate
#endif
	S65_Send_CMD_Word(0x0401); // 8Bit Color-Mode
	S65_Send_CMD_Word(0xE802); // 8Bit Transfer-Mode
	S65_Clear();
}

#endif
#endif

