/*
Copyright (C) 2008-2009  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include <stdint.h>
#include <string.h>
#include <rc.h>
#include <rc_shiftreg.h>
#include <rc_ppm-encoder.h>
#include <rc_adc.h>
#include <rc_control.h>
#include <rc_systick.h>
#include <rc_storage.h>
#include <spi.h>
#include <sd.h>
#include <draw2d.h>
#include <sound.h>
#include <lcd.h>
#include <system.h>
#include <usart.h>
#include <misc.h>


void init_switches (void);
extern void menu_init (void);
extern void menu_hw_init (void);
extern void menu (void);
extern void switches_init (void);
extern void switches_check (void);
extern void print_control_value (uint8_t value);
char Rc_REncoderDecode_menu_byte (uint8_t byte, uint8_t bit_a, uint8_t bit_b, uint8_t bit_sw);
char *int2str(char *str, int value, char mode);


int main (void) {
	uint8_t num = 0;
	uint16_t offset = 0;
	int8_t last_control_value[255];

	for (num = 0; num < 250; num++) {
		last_control_value[num] = 0;
	}


	system_init();
	init_systick();
	sound_init();
	Rc_Cmd_Init();
	Rc_AdcInit();
	Rc_ControlInit();
#ifdef SUPPORT_PPM1
	Rc_PPMEncoder1Init();
#endif
#ifdef SUPPORT_PPM2
	Rc_PPMEncoder2Init();
#endif
#ifdef SUPPORT_SPI
	SPI_Setup();
#endif
#ifdef SUPPORT_LCD
	DISPLAY_Init();
#ifdef SUPPORT_3D
	Draw3D_Init(0, 0, DISPLAY_W, DISPLAY_H);
#endif
#ifdef DISPLAY_WELLCOME_MSG
	uint8_t x;
	uint8_t y = DISPLAY_H;
	for (x = DISPLAY_W; x > 0; x -= (DISPLAY_W / 16)) {
		Draw2D_Clear();
		Draw2D_Icon(x + 1, y + 1, 86, 29, logo_rcos);
		Draw2D_Icon(DISPLAY_W - x - LOGO_VERSION_W - 1, DISPLAY_H - y - LOGO_VERSION_H - 1, LOGO_VERSION_W, LOGO_VERSION_H, logo_version, LCD_XOR);
		y -= (DISPLAY_H / 16);
		DISPLAY_Update();
	}
	sound(640 + y * 10, 10);
	MotionTimer = 0; while (MotionTimer < 250) {}
	for (x = 0; x < DISPLAY_W; x += (DISPLAY_W / 4)) {
		Draw2D_Clear();
		Draw2D_Icon(x + 1, y + 15, 86, 29, logo_rcos);
		Draw2D_Icon(DISPLAY_W - x - LOGO_VERSION_W - 1, DISPLAY_H - LOGO_VERSION_H - 1, LOGO_VERSION_W, LOGO_VERSION_H, logo_version, LCD_XOR);
		DISPLAY_Update();
	}
	MotionTimer = 0; while (MotionTimer < 250) {}
#endif
#endif
#ifdef RCOS_MAINBOARD_V1_0
	BootMSG(60, "Switches-Init");
	switches_init();
	Rc_ShiftRegInit();
#endif
#ifdef SUPPORT_I2C
	BootMSG(70, "I2C-Init");
	Rc_I2CInit();
#endif
#ifdef SUPPORT_MENU
	BootMSG(90, "Menu-Init");
	menu_init();
	menu_hw_init();
	menu();
#endif

	display_message_str[0][0] = 0;
	display_message_str[1][0] = 0;
	display_control = 0;
	Last_Control = 255;

#ifdef SUPPORT_USART
	init_usart();
#endif
#ifdef SUPPORT_MMC
	rc_mmc_init();
#endif
#ifdef SUPPORT_PPM1
	BootMSG(80, "PPM1-Power");
	Rc_PPMEncoder1Power(1);
#endif
#ifdef SUPPORT_PPM2
	BootMSG(80, "PPM2-Power");
	Rc_PPMEncoder2Power(1);
#endif

	while (1) {
		// Beep
		if (beeper_count > 0) {
			beeper_count--;
			sound(beeper_freq, beeper_len);
		}

		// Servo-Test's
		if (Servo_Testflag != 0) {
			num = Servo_Testflag - 1;
			Servo_Testflag = 0;

#ifdef SUPPORT_PPM1
			if (num < PPM1_CHANNELS_MAX) {
				Rc_PPMEncoder1Set(num, -1000);
#ifdef SUPPORT_PPM2
			} else if (num - PPM1_CHANNELS_MAX < PPM2_CHANNELS_MAX) {
				Rc_PPMEncoder2Set(num - PPM1_CHANNELS_MAX, -1000);
#endif
			}
			MotionTimer = 0; while (MotionTimer < 250) {}
			MotionTimer = 0; while (MotionTimer < 250) {}
			MotionTimer = 0; while (MotionTimer < 250) {}
			MotionTimer = 0; while (MotionTimer < 250) {}
			MotionTimer = 0; while (MotionTimer < 250) {}
			MotionTimer = 0; while (MotionTimer < 250) {}
#endif
#ifdef SUPPORT_PPM1
			if (num < PPM1_CHANNELS_MAX) {
				Rc_PPMEncoder1Set(num, 1000);
#ifdef SUPPORT_PPM2
			} else if (num - PPM1_CHANNELS_MAX < PPM2_CHANNELS_MAX) {
				Rc_PPMEncoder2Set(num - PPM1_CHANNELS_MAX, 1000);
#endif
			}
			MotionTimer = 0; while (MotionTimer < 250) {}
			MotionTimer = 0; while (MotionTimer < 250) {}
			MotionTimer = 0; while (MotionTimer < 250) {}
			MotionTimer = 0; while (MotionTimer < 250) {}
			MotionTimer = 0; while (MotionTimer < 250) {}
			MotionTimer = 0; while (MotionTimer < 250) {}
#endif
		} else {

#ifdef SUPPORT_USART
#ifdef SUPPORT_FMS
			static uint8_t fms_count = 0;
			if (fms_output == 1 && fms_count < SystemTimer - 20 || fms_count > SystemTimer) {
				fms_count = SystemTimer;
				rc_fms_serial_send();
			}
#else
			// Serial-Response
			switch (Tx1_Flag) {
				case FE_CMD_CONTROL_GET:
					NbrOfDataToTransfer[0] = Rc_Cmd_Control_Get((uint8_t *)TxBuffer[0]);
					NbrOfDataToTransfer[0] = add_csum((uint8_t *)TxBuffer[0], NbrOfDataToTransfer[0]);
					USART_TX_START(0);
				break;
				case FE_CMD_MIXER_GET:
					NbrOfDataToTransfer[0] = Rc_Cmd_Mixer_Get((uint8_t *)TxBuffer[0]);
					NbrOfDataToTransfer[0] = add_csum((uint8_t *)TxBuffer[0], NbrOfDataToTransfer[0]);
					USART_TX_START(0);
				break;
				case FE_CMD_OUTPUT_GET:
					NbrOfDataToTransfer[0] = Rc_Cmd_Output_Get((uint8_t *)TxBuffer[0]);
					NbrOfDataToTransfer[0] = add_csum((uint8_t *)TxBuffer[0], NbrOfDataToTransfer[0]);
					USART_TX_START(0);
				break;
				case FE_CMD_DUALRATE_GET:
					NbrOfDataToTransfer[0] = Rc_Cmd_Dualrate_Get((uint8_t *)TxBuffer[0]);
					NbrOfDataToTransfer[0] = add_csum((uint8_t *)TxBuffer[0], NbrOfDataToTransfer[0]);
					USART_TX_START(0);
				break;
				case FE_CMD_EXPO_GET:
					NbrOfDataToTransfer[0] = Rc_Cmd_Expo_Get((uint8_t *)TxBuffer[0]);
					NbrOfDataToTransfer[0] = add_csum((uint8_t *)TxBuffer[0], NbrOfDataToTransfer[0]);
					USART_TX_START(0);
				break;
				case FE_CMD_MOTION_GET:
					NbrOfDataToTransfer[0] = Rc_Cmd_Motion_Get((uint8_t *)TxBuffer[0]);
					NbrOfDataToTransfer[0] = add_csum((uint8_t *)TxBuffer[0], NbrOfDataToTransfer[0]);
					USART_TX_START(0);
				break;
				case FE_CMD_LIMITER_GET:
					NbrOfDataToTransfer[0] = Rc_Cmd_Limiter_Get((uint8_t *)TxBuffer[0]);
					NbrOfDataToTransfer[0] = add_csum((uint8_t *)TxBuffer[0], NbrOfDataToTransfer[0]);
					USART_TX_START(0);
				break;
				case FE_CMD_ATV_GET:
					NbrOfDataToTransfer[0] = Rc_Cmd_Atv_Get((uint8_t *)TxBuffer[0]);
					NbrOfDataToTransfer[0] = add_csum((uint8_t *)TxBuffer[0], NbrOfDataToTransfer[0]);
					USART_TX_START(0);
				break;
				case FE_CMD_PPMENCODER1_MENCODER_GET:
					NbrOfDataToTransfer[0] = Rc_Cmd_PPMEncoder1_Mencoder_Get((uint8_t *)TxBuffer[0]);
					NbrOfDataToTransfer[0] = add_csum((uint8_t *)TxBuffer[0], NbrOfDataToTransfer[0]);
					USART_TX_START(0);
				break;
				case FE_CMD_PPMENCODER1_GET:
					NbrOfDataToTransfer[0] = Rc_Cmd_PPMEncoder1_Get((uint8_t *)TxBuffer[0]);
					NbrOfDataToTransfer[0] = add_csum((uint8_t *)TxBuffer[0], NbrOfDataToTransfer[0]);
					USART_TX_START(0);
				break;
				case FE_CMD_PPMENCODER2_GET:
					NbrOfDataToTransfer[0] = Rc_Cmd_PPMEncoder2_Get((uint8_t *)TxBuffer[0]);
					NbrOfDataToTransfer[0] = add_csum((uint8_t *)TxBuffer[0], NbrOfDataToTransfer[0]);
					USART_TX_START(0);
				break;
				case FE_CMD_ACT_FREQ_GET:
					Act_Cmd_Send(ACT_DSL_GET_FREQTABLE);
				break;
				case FE_CMD_ACT_INFO_GET:
					Act_Cmd_Send(ACT_DSL_GET_INFO);
				break;
				case FE_CMD_CONTROL_VALUE_SET:
					ControlWrite(RxBuffer[0][0], RxBuffer[0][1]);
					USART_SEND_OK(0);
				break;
				case FE_CMD_CONTROL_SET:
					Rc_Cmd_Control_Set((uint8_t *)RxBuffer[0]);
					USART_SEND_OK(0);
				break;
				case FE_CMD_PPMENCODER1_MENCODER_SET:
					Rc_Cmd_PPMEncoder1_Mencoder_Set((uint8_t *)RxBuffer[0]);
					USART_SEND_OK(0);
				break;
				case FE_CMD_PPMENCODER1_SET:
					Rc_Cmd_PPMEncoder1_Set((uint8_t *)RxBuffer[0]);
					USART_SEND_OK(0);
				break;
				case FE_CMD_PPMENCODER2_SET:
					Rc_Cmd_PPMEncoder2_Set((uint8_t *)RxBuffer[0]);
					USART_SEND_OK(0);
				break;
				case FE_CMD_OUTPUT_SET:
					Rc_Cmd_Output_Set((uint8_t *)RxBuffer[0]);
					USART_SEND_OK(0);
				break;
				case FE_CMD_EXPO_SET:
					Rc_Cmd_Expo_Set((uint8_t *)RxBuffer[0]);
					USART_SEND_OK(0);
				break;
				case FE_CMD_LIMITER_SET:
					Rc_Cmd_Limiter_Set((uint8_t *)RxBuffer[0]);
					USART_SEND_OK(0);
				break;
				case FE_CMD_ATV_SET:
					Rc_Cmd_Atv_Set((uint8_t *)RxBuffer[0]);
					USART_SEND_OK(0);
				break;
				case FE_CMD_DUALRATE_SET:
					Rc_Cmd_Dualrate_Set((uint8_t *)RxBuffer[0]);
					USART_SEND_OK(0);
				break;
				case FE_CMD_MOTION_SET:
					Rc_Cmd_Motion_Set((uint8_t *)RxBuffer[0]);
					USART_SEND_OK(0);
				break;
				case FE_CMD_MIXER_SET:
					Rc_Cmd_Mixer_Set((uint8_t *)RxBuffer[0]);
					USART_SEND_OK(0);
				break;

				case 253:
					USART_SEND_OK(0);
				break;
				case 254:
					USART1_SEND_STR("ERROR:CSUM\n");
				break;
				case 255:
					USART1_SEND_STR("ERROR:CSUM\n");
				break;
			}
			Tx1_Flag = 0;
#endif
#endif

			// Update Control-Values's
#ifdef RCOS_MAINBOARD_V1_0
			switches_check();
			uint8_t enc_data[3];
			Rc_ShiftRegRead_Data(enc_data, 2);
			Rc_REncoderDecode_byte(enc_data[0], 3, 4, 2, 20);
			Rc_REncoderDecode_byte(enc_data[0], 5, 6, 7, 21);
			Rc_REncoderDecode_byte(enc_data[1], 3, 4, 2, 22);
			Rc_REncoderDecode_byte(enc_data[1], 5, 6, 7, 23);
#endif

			// Mixer-Stages's
			Rc_Stage_Input();
			Rc_Stage_Mixer();
			Rc_Stage_Output();

			// Update Output-Value's (for PPM, ...)
#ifdef SUPPORT_PPM1
			offset = 0;
			for (num = 0; num < PPM1_CHANNELS_MAX; num++) {
				Rc_PPMEncoder1Set(num, Rc_Output_value[num + offset]);
			}
#endif
#ifdef SUPPORT_PPM2
			offset = PPM1_CHANNELS_MAX;
			for (num = 0; num < PPM2_CHANNELS_MAX; num++) {
				Rc_PPMEncoder2Set(num, Rc_Output_value[num + offset]);
			}
#endif
#ifdef SUPPORT_MULTIPROP
			offset = PPM1_CHANNELS_MAX + PPM2_CHANNELS_MAX;
			for (num2 = 0; num2 < PPM1_MENCODER_MAX; num2++) {
				for (num = 0; num < PPM1_MENCODER_CHANNEL_MAX; num++) {
					Rc_PPMEncoder1_Mencoder_values[num2][num] = Rc_Output_value[num + offset];
				}
				offset += PPM1_MENCODER_CHANNEL_MAX;
			}
#endif
		}

		// Display Messages/Control-Changes on LCD
#ifdef SUPPORT_LCD
		if (control_change_popup == 1 && lcd_count++ > 250) {
			lcd_count = 0;
			for (num = 0; num < 250; num++) {
				int16_t val = (int16_t)ControlRead(num);
				if (num > ADC_CHANNELS_MAX && num < 50 && val != last_control_value[num]) {
					display_control = num;
					lcd_msg_time = 600;
					Last_Control = num;
				} else if (num <= ADC_CHANNELS_MAX && (val - 20 > last_control_value[num] || val + 20 < last_control_value[num])) {
					if (num >= 4) {
						display_control = num;
						lcd_msg_time = 600;
					}
					Last_Control = num;
				}
				last_control_value[num] = val;
			}
		}
		if (lcd_msg_time > 0) {
			lcd_msg_time--;
			if (display_control != 0 || display_message_str[0][0] != 0 || display_message_str[1][0] != 0) {
				if (display_control != 0) {
#ifdef SUPPORT_MENU
					print_control_value(display_control);
#endif
					Last_Control = display_control;
				} else {
					Draw2D_StringCenter(0, (char *)display_message_str[0], LCD_XOR, LCD_TRANS);
					Draw2D_StringCenter(16, (char *)display_message_str[1], LCD_XOR, LCD_TRANS);
				}
				display_message_str[0][0] = 0;
				display_message_str[1][0] = 0;
				display_control = 0;
				DISPLAY_Update();
			}
		} else {
#ifdef SUPPORT_MENU
			menu();
#endif
		}
#endif

	}
}



