
#include <stdint.h>
#include <string.h>
#include <rc.h>
#include <draw2d.h>
#include <rc_adc.h>
#include <rc_control.h>
#include <rc_menu_support.h>
#include <rc_storage.h>

#ifdef SUPPORT_MENU

const char *name_ok = "Ok";
const char *name_back = "Back";
static const char *name_aileron = "Querruder";
static const char *name_aileron1 = "Querruder L";
static const char *name_aileron2 = "Querruder R";
static const char *name_elevator = "Hohenruder";
static const char *name_elevator1 = "Hohenruder L";
static const char *name_elevator2 = "Hohenruder R";
static const char *name_rudder = "Seitenruder";
static const char *name_throttle = "Drossel";
static const char *name_flaps = "Landeklappen";
static const char *name_nick = "Nick";
static const char *name_pitch = "Pitch";
static const char *name_roll = "Roll";
static const char *name_roll1 = "Roll-Links";
static const char *name_roll2 = "Roll-Rechts";

extern uint8_t redraw;

#ifdef PLATFORM_SDL
extern uint16_t joy_value[20];
#endif

uint8_t menu_get_cmd (void) {
	static int8_t encoder = 0;
	uint8_t enc_data = 0;
	uint8_t cmd = CMD_NONE;
#ifdef PLATFORM_SDL
	SDL_Event event;
	cmd = CMD_NONE;
	while ( SDL_PollEvent(&event) ) {
		switch (event.type) {
			case SDL_JOYAXISMOTION:
				if (event.jaxis.axis == 0) {
					joy_value[3] = (event.jaxis.value / 16) + 2048;
				} else if (event.jaxis.axis == 1) {
					joy_value[2] = (event.jaxis.value / 16) + 2048;
				} else if (event.jaxis.axis == 2) {
					joy_value[0] = (event.jaxis.value / 16) + 2048;
				} else if (event.jaxis.axis == 3) {
					joy_value[1] = (event.jaxis.value / 16) + 2048;
				} else {
					joy_value[event.jaxis.axis] = (event.jaxis.value / 16) + 2048;
				}
				//printf("Joystick %d axis %d value: %d %d\n", event.jaxis.which, event.jaxis.axis, (event.jaxis.value / 16) + 2048, (event.jaxis.value / 16) + 2048);
			break;
			case SDL_JOYBUTTONDOWN:
				Rc_Control_value[30 + event.jbutton.button] = 127 - Rc_Control_value[30 + event.jbutton.button];
				//printf("Joystick %d button %d down\n", event.jbutton.which, event.jbutton.button);
			break;
//			case SDL_JOYBUTTONUP:
//				Rc_Control_value[30 + event.jbutton.button] = 0;
//				printf("Joystick %d button %d up\n", event.jbutton.which, event.jbutton.button);
//			break;
			case SDL_MOUSEBUTTONDOWN:
				if (event.button.button == 5) {
					cmd = CMD_UP;
				} else if (event.button.button == 4) {
					cmd = CMD_DOWN;
				} else if (event.button.button == 1 || event.button.button == 2) {
					cmd = CMD_ENTER;
				} else if (event.button.button == 3) {
					cmd = CMD_BACK;
				}
			break;
			case SDL_QUIT:
				exit(0);
			break;
		}
	}
#endif
#ifdef PLATFORM_STM32
	if (GPIO_ReadInputDataBit(RC_CONTROL_SW0_A_PORT, RC_CONTROL_SW0_A_PIN) != 0) {
		enc_data |= 1;
	}
	if (GPIO_ReadInputDataBit(RC_CONTROL_SW0_B_PORT, RC_CONTROL_SW0_B_PIN) != 0) {
		enc_data |= 2;
	}
	if (GPIO_ReadInputDataBit(RC_CONTROL_SW0_C_PORT, RC_CONTROL_SW0_C_PIN) != 0) {
		enc_data |= 4;
	}
	cmd = Rc_REncoderDecode_menu_byte(enc_data, 0, 1, 2);
	if (cmd == CMD_UP) {
		encoder++;
		if (encoder >= 4) {
			encoder = 0;
			sound(1000, 20);
		} else {
			cmd = 0;
		}
	} else if (cmd == CMD_DOWN) {
		encoder--;
		if (encoder <= -5) {
			encoder = 0;
			sound(1200, 20);
		} else {
			cmd = 0;
		}
	} else if (cmd == CMD_ENTER) {
		sound(1300, 20);
		uint16_t timeout = 60000;
		while (cmd == CMD_ENTER && timeout > 0) {
			if (GPIO_ReadInputDataBit(RC_CONTROL_SW0_A_PORT, RC_CONTROL_SW0_A_PIN) != 0) {
				enc_data |= 1;
			}
			if (GPIO_ReadInputDataBit(RC_CONTROL_SW0_B_PORT, RC_CONTROL_SW0_B_PIN) != 0) {
				enc_data |= 2;
			}
			if (GPIO_ReadInputDataBit(RC_CONTROL_SW0_C_PORT, RC_CONTROL_SW0_C_PIN) != 0) {
				enc_data |= 4;
			}
			cmd = Rc_REncoderDecode_menu_byte(enc_data, 0, 1, 2);
			timeout--;
		}
		if (timeout == 0) {
			sound(1000, 20);
			sound(1100, 20);
			sound(1200, 20);
			sound(1300, 20);
			sound(1400, 20);
			sound(500, 20);
			sound(600, 20);
			sound(700, 20);
			sound(800, 20);
			sound(900, 20);
			while (cmd == CMD_ENTER) {
				if (GPIO_ReadInputDataBit(RC_CONTROL_SW0_A_PORT, RC_CONTROL_SW0_A_PIN) != 0) {
					enc_data |= 1;
				}
				if (GPIO_ReadInputDataBit(RC_CONTROL_SW0_B_PORT, RC_CONTROL_SW0_B_PIN) != 0) {
					enc_data |= 2;
				}
				if (GPIO_ReadInputDataBit(RC_CONTROL_SW0_C_PORT, RC_CONTROL_SW0_C_PIN) != 0) {
					enc_data |= 4;
				}
				cmd = Rc_REncoderDecode_menu_byte(enc_data, 0, 1, 2);
			}
			cmd = CMD_BACK;
		} else {
			sound(1400, 20);
			cmd = CMD_ENTER;
		}
	}
#endif
	return cmd;
}


void print_mixer (uint8_t offset_x, uint8_t offset_y, uint8_t width, uint8_t height, uint8_t id, uint8_t point, uint8_t xy) {
	uint8_t n;
	uint8_t x;
	uint8_t y;
	uint8_t last_x = 0;
	uint8_t last_y = 0;

	if (xy != 2) {
		for (n = 0; n < MIXER_POINTS_MAX; n++) {
			char tmp_str[10];
			if (n == point) {
				if (xy == 0) {
					Draw2D_PrintXY(offset_x, offset_y + n * 6, ">", LCD_XOR, LCD_TRANS);
				} else {
					Draw2D_PrintXY(offset_x + 18, offset_y + n * 6, ">", LCD_XOR, LCD_TRANS);
				}
			}
			Draw2D_PrintXY(offset_x + 5, offset_y + n * 6, (char *)int2str((char *)tmp_str, Rc_Mixer_point[id][n][1], 0), LCD_XOR, LCD_TRANS);
			Draw2D_PrintXY(offset_x + 23, offset_y + n * 6, (char *)int2str((char *)tmp_str, Rc_Mixer_point[id][n][0], 0), LCD_XOR, LCD_TRANS);
		}
	}

	offset_x += 35;
	width -= 35;

	for (n = offset_x; n < offset_x + width; n++) {
		Draw2D_Pixel(n, offset_y, 1);
		Draw2D_Pixel(n, offset_y + height - 1, 1);
	}
	for (n = offset_y; n < offset_y + height; n++) {
		Draw2D_Pixel(offset_x, n, 1);
		Draw2D_Pixel(offset_x + width - 1, n, 1);
	}
	x = Rc_Mixer_point[id][0][0] * (width - 2) / 100 + 1;
	y = Rc_Mixer_point[id][0][1] * (height - 2) / 100 + 1;

	last_x = x;
	last_y = y;
	for (n = 0; n < MIXER_POINTS_MAX; n++) {
		x = Rc_Mixer_point[id][n][0] * (width - 2) / 100 + 1;
		y = (Rc_Mixer_point[id][n][1] * (height - 2) / 100 + 1);
		y = (y - (height / 2)) * ControlSet(Rc_Mixer_percent_control[id], Rc_Mixer_percent_value[id]) / 100 + (height / 2);
		if (n == point) {
			if (xy == 1) {
				Draw2D_Pixel(offset_x + x + 6, offset_y + y, LCD_FG);
				Draw2D_Pixel(offset_x + x - 6, offset_y + y, LCD_FG);
			} else {
				Draw2D_Pixel(offset_x + x, offset_y + y + 6, LCD_FG);
				Draw2D_Pixel(offset_x + x, offset_y + y - 6, LCD_FG);
			}
			Draw2D_Pixel(offset_x + x + 4, offset_y + y, LCD_FG);
			Draw2D_Pixel(offset_x + x + 3, offset_y + y, LCD_FG);
			Draw2D_Pixel(offset_x + x + 2, offset_y + y, LCD_FG);
			Draw2D_Pixel(offset_x + x + 1, offset_y + y, LCD_FG);
			Draw2D_Pixel(offset_x + x - 1, offset_y + y, LCD_FG);
			Draw2D_Pixel(offset_x + x - 2, offset_y + y, LCD_FG);
			Draw2D_Pixel(offset_x + x - 3, offset_y + y, LCD_FG);
			Draw2D_Pixel(offset_x + x - 4, offset_y + y, LCD_FG);
			Draw2D_Pixel(offset_x + x, offset_y + y + 4, LCD_FG);
			Draw2D_Pixel(offset_x + x, offset_y + y + 3, LCD_FG);
			Draw2D_Pixel(offset_x + x, offset_y + y + 2, LCD_FG);
			Draw2D_Pixel(offset_x + x, offset_y + y + 1, LCD_FG);
			Draw2D_Pixel(offset_x + x, offset_y + y - 1, LCD_FG);
			Draw2D_Pixel(offset_x + x, offset_y + y - 2, LCD_FG);
			Draw2D_Pixel(offset_x + x, offset_y + y - 3, LCD_FG);
			Draw2D_Pixel(offset_x + x, offset_y + y - 4, LCD_FG);
		}
		Draw2D_Line(offset_x + last_x, offset_y + last_y, offset_x + x, offset_y + y, LCD_FG);
		last_x = x;
		last_y = y;
	}
	x = (Rc_Control_buffer[Rc_Mixer_channel_in[id]] / 8 + 127) * (width - 2) / 255 + 1;
	y = (Rc_Output_value[Rc_Mixer_channel_out1[id]] / 8 + 127) * (height - 2) / 255 + 1;
	Draw2D_Line(offset_x + x, offset_y, offset_x + x, offset_y + height, LCD_FG);
	Draw2D_Line(offset_x, offset_y + y, offset_x + width, offset_y + y, LCD_FG);
	y = ControlSet(Rc_Mixer_percent_control[id], Rc_Mixer_percent_value[id]) * (height - 2) / 200;
	for (n = offset_x; n < offset_x + width; n += 2) {
		Draw2D_Pixel(n, offset_y + (height / 2) + y - 1, LCD_FG);
		Draw2D_Pixel(n, offset_y + (height / 2) - y - 1, LCD_FG);
	}
	for (n = 0; n < MIXER_POINTS_MAX; n++) {
		x = Rc_Mixer_point[id][n][0] * (width - 2) / 100 + 1;
		y = (Rc_Mixer_point[id][n][1] * (height - 2) / 100 + 1);
		y = (y - (height / 2)) * ControlSet(Rc_Mixer_percent_control[id], Rc_Mixer_percent_value[id]) / 100 + (height / 2);
		if (n == point) {
			char tmp_str[10];
			if (y < (height / 2)) {
				Draw2D_Box(offset_x + x, offset_y + y, 20, 7);
				Draw2D_PrintXY(offset_x + x + 1, offset_y + y + 1, (char *)int2str((char *)tmp_str, Rc_Mixer_point[id][n][0], 0), LCD_XOR, LCD_TRANS);
				Draw2D_PrintXY(offset_x + x + 12, offset_y + y + 1, (char *)int2str((char *)tmp_str, Rc_Mixer_point[id][n][1], 0), LCD_XOR, LCD_TRANS);
			} else {
				Draw2D_Box(offset_x + x - 20, offset_y + y - 7, 20, 7);
				Draw2D_PrintXY(offset_x + x + 1 - 20, offset_y + y + 1 - 7, (char *)int2str((char *)tmp_str, Rc_Mixer_point[id][n][0], 0), LCD_XOR, LCD_TRANS);
				Draw2D_PrintXY(offset_x + x + 12 - 20, offset_y + y + 1 - 7, (char *)int2str((char *)tmp_str, Rc_Mixer_point[id][n][1], 0), LCD_XOR, LCD_TRANS);
			}
			break;
		}
	}
	Draw2D_Font_SetNum(1);
}

void print_bar (uint8_t offset_x, uint8_t offset_y, uint8_t width, uint8_t height, uint8_t value) {
	uint8_t n;
	Draw2D_Line(offset_x, offset_y, offset_x + width, offset_y, LCD_XOR);
	Draw2D_Line(offset_x, offset_y + height, offset_x + width, offset_y + height, LCD_XOR);
	Draw2D_Line(offset_x, offset_y, offset_x, offset_y + height, LCD_XOR);
	Draw2D_Line(offset_x + width, offset_y, offset_x + width, offset_y + height, LCD_XOR);
	Draw2D_Line(offset_x + (value * width / 255), offset_y, offset_x + (value * width / 255), offset_y + height, LCD_XOR);
	for (n = offset_y; n < offset_y + height; n += 2) {
		Draw2D_Pixel(offset_x + (width / 2), n, LCD_XOR);
	}
}

uint8_t print_control (uint8_t x, uint8_t y, uint8_t control_id, uint8_t fg, uint8_t bg) {
	uint8_t tmp_str[10];
	uint8_t x2;
	if (Rc_Control_name[control_id][0] != 0) {
		x = Draw2D_PrintXY(x, y, (char *)Rc_Control_name[control_id], fg, bg);
	} else {
		x = Draw2D_PrintXY(x, y, (char *)int2str((char *)tmp_str, control_id, 0), fg, bg);
	}
	x = Draw2D_PrintXY(x + 2, y, (char *)int2str_new((char *)tmp_str, ControlRead(control_id), 0, 0, 1), fg, bg);
	x2 = DISPLAY_W - 30;
	if (control_id < 4) {
		Draw2D_Circle(x2, y + 3, 4, fg);
		Draw2D_Circle(x2 + 12, y + 3, 4, fg);
		if (control_id == 0) {
			Draw2D_Line(x2 + 12 - 3, y + 3, x2 + 12 + 3, y + 3, fg);
		} else if (control_id == 1) {
			Draw2D_Line(x2 + 12, y + 3 - 3, x2 + 12, y + 3 + 3, fg);
		} else if (control_id == 2) {
			Draw2D_Line(x2, y + 3 - 3, x2, y + 3 + 3, fg);
		} else if (control_id == 3) {
			Draw2D_Line(x2 - 3, y + 3, x2 + 3, y + 3, fg);
		}
	}
	return x;
}

void print_point (uint8_t x, uint8_t y, uint8_t control_id, uint8_t fg, uint8_t bg) {
	Draw2D_PrintXY(x, y, (char *)"->", fg, bg);
}

void print_time (uint8_t x, uint8_t y, uint32_t timer_count, uint8_t fg, uint8_t bg) {
	uint8_t tmp_str[20];
	uint8_t pos_x = x;
	int2str_new((char *)tmp_str, (timer_count / 1000) / 60, 1, 0, 1);
	pos_x = Draw2D_PrintXY(pos_x, y, (char *)tmp_str, fg, bg);
	pos_x = Draw2D_PrintXY(pos_x, y, (char *)":", fg, bg);
	int2str_new((char *)tmp_str, (uint16_t)((timer_count / 1000) - ((timer_count / 1000) / 60 * 60)), 1, 0, 1);
	pos_x = Draw2D_PrintXY(pos_x, y, (char *)tmp_str, fg, bg);
}

void print_control_value (uint8_t control_id) {
	int16_t new_value = ControlRead(control_id) * (DISPLAY_W - 20) / 255;
	Draw2D_Box(5, (DISPLAY_H / 2) - 25, DISPLAY_W - 10, 50);
	Draw2D_Rec(3, (DISPLAY_H / 2) - 27, DISPLAY_W - 6, 54);
	Draw2D_Font_SetNum(2);
	print_control(255, (DISPLAY_H / 2) - 20, control_id, LCD_XOR, LCD_TRANS);
	Draw2D_Font_SetNum(1);
	Draw2D_BoxInverted(10, (DISPLAY_H / 2) - 5, DISPLAY_W - 20, 10);
	if (new_value > 0) {
		Draw2D_Box((DISPLAY_W / 2), (DISPLAY_H / 2) - 4, new_value, 8);
	} else {
		Draw2D_Box((DISPLAY_W / 2) - (new_value * -1), (DISPLAY_H / 2) - 4, (new_value * -1), 8);
	}
}

uint8_t print_output (uint8_t x, uint8_t y, uint8_t output_id, uint8_t fg, uint8_t bg) {
	uint8_t tmp_str[10];
	if (output_id >= OUTPUT_MAX) {
		x = Draw2D_PrintXY(x, y, (char *)"Unbelegt", fg, bg);
	} else {
		if (Rc_Output_name[output_id][0] != 0) {
			x = Draw2D_PrintXY(x, y, (char *)Rc_Output_name[output_id], fg, bg);
		} else {
			x = Draw2D_PrintXY(x, y, (char *)int2str((char *)tmp_str, output_id, 0), fg, bg);
		}
		x = Draw2D_PrintXY(x + 2, y, (char *)int2str_new((char *)tmp_str, Rc_Output_value[output_id] / 8, 0, 0, 1), fg, bg);
	}
	return x;
}

void menu_hw_init (void) {
#ifdef PLATFORM_STM32
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Pin = 0 | RC_CONTROL_SW0_A_PIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(RC_CONTROL_SW0_A_PORT, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = 0 | RC_CONTROL_SW0_B_PIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(RC_CONTROL_SW0_B_PORT, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = 0 | RC_CONTROL_SW0_C_PIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(RC_CONTROL_SW0_C_PORT, &GPIO_InitStructure);
#endif
}

uint8_t menu_control_overview (uint8_t cmd) {
	static uint8_t entry = 0;
	uint8_t tmp_str[40];
	uint8_t num = 0;
	uint8_t x = 0;

	if (cmd == CMD_ENTER) {
		redraw = 2;
		return 1;
	} else if (cmd == CMD_UP && entry < 16) {
		entry++;
	} else if (cmd == CMD_DOWN && entry > 0) {
		entry--;
	}

	Draw2D_Clear();
	DISPLAY_Header((char *)Rc_Model_name);
	x = Draw2D_PrintXY(0, 10, (char *)int2str((char *)tmp_str, (entry * 10) + 1, 0), LCD_XOR, LCD_TRANS);
	x = Draw2D_PrintXY(x + 2, 10, "->", LCD_XOR, LCD_TRANS);
	Draw2D_PrintXY(x + 2, 10, (char *)int2str((char *)tmp_str, (entry * 10) + 10, 0), LCD_XOR, LCD_TRANS);
//	for (num = 0; num < 4; num++) {
//		print_bar(3, 16 + num * ((DISPLAY_H - 16) / 4) + 1, DISPLAY_W - 6, ((DISPLAY_H - 16) / 4) - 2, (Rc_AdcRead((entry * 4) + num) / 8) + 128);
//	}
	for (num = 0; num < 10; num++) {
		print_bar(3, 16 + num * ((DISPLAY_H - 16) / 10) + 1, DISPLAY_W - 6, ((DISPLAY_H - 16) / 10) - 2, ((int16_t)ControlRead((entry * 10) + num) / 2) + 128);
	}
	return 0;
}

uint8_t menu_output_overview (uint8_t cmd) {
	static uint8_t entry = 0;
	uint8_t tmp_str[40];
	uint8_t num = 0;
	uint8_t x = 0;

	if (cmd == CMD_ENTER) {
		redraw = 2;
		return 1;
	} else if (cmd == CMD_UP && entry < 5) {
		entry++;
	} else if (cmd == CMD_DOWN && entry > 0) {
		entry--;
	}

	Draw2D_Clear();
	DISPLAY_Header((char *)Rc_Model_name);
	x = Draw2D_PrintXY(0, 10, (char *)int2str((char *)tmp_str, (entry * 4) + 1, 0), LCD_XOR, LCD_TRANS);
	x = Draw2D_PrintXY(x + 2, 10, "->", LCD_XOR, LCD_TRANS);
	Draw2D_PrintXY(x + 2, 10, (char *)int2str((char *)tmp_str, (entry * 4) + 7, 0), LCD_XOR, LCD_TRANS);
	for (num = 0; num < 4; num++) {
		print_bar(3, 16 + num * ((DISPLAY_H - 16) / 4) + 1, DISPLAY_W - 6, ((DISPLAY_H - 16) / 4) - 2, (Rc_Output_value[(entry * 4) + num] / 8) + 128);
	}
	return 0;
}

uint8_t menu_calibrate (uint8_t cmd) {
	static uint8_t entry = 0;
	int8_t num = 0;
	if (cmd == CMD_ENTER) {
		redraw = 2;
		if (entry == 0) {
			entry++;
			for (num = 0; num < 4; num++) {
				Rc_Calibrate_center[num] = 0;
				Rc_Calibrate_min[num] = -1024;
				Rc_Calibrate_max[num] = 1024;
			}
			MotionTimer = 0; while (MotionTimer < 250) {}
			MotionTimer = 0; while (MotionTimer < 250) {}
			for (num = 0; num < 4; num++) {
				Rc_Calibrate_center[num] = Rc_AdcRead(num) * -1;
				Rc_Calibrate_min[num] = 1024;
				Rc_Calibrate_max[num] = -1024;
			}
		} else if (entry == 1) {
			return 1;
		}
	}


	Draw2D_Clear();
	DISPLAY_Header((char *)Rc_Model_name);
	Draw2D_PrintXY(0, 10, (char *)"Calibrate:", LCD_XOR, LCD_TRANS);

	Draw2D_Font_SetNum(2);

	if (entry == 0) {
		Draw2D_StringCenter(20, "Center", LCD_XOR, LCD_TRANS);
		Draw2D_StringCenter(30, "all", LCD_XOR, LCD_TRANS);
		Draw2D_StringCenter(40, "sticks", LCD_XOR, LCD_TRANS);
	} else if (entry == 1) {
		Draw2D_StringCenter(20, "Move sticks", LCD_XOR, LCD_TRANS);
		Draw2D_StringCenter(30, "to each", LCD_XOR, LCD_TRANS);
		Draw2D_StringCenter(40, "korner", LCD_XOR, LCD_TRANS);
	} else {
		Draw2D_StringCenter(50, "> DONE <", LCD_XOR, LCD_TRANS);
	}

	Draw2D_Font_SetNum(1);

	return 0;
}

uint8_t menu_adc (uint8_t cmd) {
	static uint8_t entry = 0;
	static uint8_t pos = 10;
	uint8_t tmp_str[40];

	if (cmd == CMD_ENTER) {
		redraw = 2;
		return 1;
	} else if (cmd == CMD_UP && entry < 5) {
		entry++;
	} else if (cmd == CMD_DOWN && entry > 0) {
		entry--;
	}

	if (pos < DISPLAY_W - 10) {
		Draw2D_Pixel(pos, 30 + ((Rc_AdcRead(entry) / 8) + 128) / 16 + 1, 1);
		pos++;
	} else {
		DISPLAY_Update();
		Draw2D_Clear();
		DISPLAY_Header((char *)Rc_Model_name);
		Draw2D_PrintXY(8, 0, "ADC-Input:     ", LCD_XOR, LCD_TRANS);
		Draw2D_PrintXY(8, 12, (char *)int2str((char *)tmp_str, entry + 1, 0), LCD_XOR, LCD_TRANS);
		pos = 10;
	}
	return 0;
}


#ifdef SUPPORT_MMC

uint8_t menu_mmc_load (uint8_t cmd) {
	static uint8_t entry = 0;
	uint8_t tmp_str[40];
	int8_t num = 0;
	int8_t x = 0;
	int16_t start_num = 0;


	if (cmd == CMD_ENTER) {
		if (entry > 0) {
			load_setup_mmc(entry - 1);
		}
		redraw = 2;
		return 1;
	} else if (cmd == CMD_UP) {
		if (entry < 10) {
			entry++;
		}
	} else if (cmd == CMD_DOWN) {
		if (entry > 0) {
			entry--;
		}
	}

	Draw2D_Clear();
	DISPLAY_Header((char *)Rc_Model_name);
	Draw2D_PrintXY(0, 10, "Load:", LCD_XOR, LCD_TRANS);

	num = 0;
	start_num = entry - MENU_HEIGHT / 2;
	if (start_num < 0) {
		start_num = 0;
	}
	for (num = 0; num < MENU_HEIGHT && num + start_num < 10; num++) {
		if (num + start_num == entry) {
			Draw2D_Box(2, 20 + (num * 10), DISPLAY_W - 13, 10);
		}

		if (num + start_num == 0) {
			x = Draw2D_PrintXY(3, 22 + (num * 10), (char *)"Back", LCD_XOR, LCD_TRANS);
		} else {
			load_name_mmc((char *)tmp_str, num + start_num - 1);
			x = Draw2D_PrintXY(3, 22 + (num * 10), (char *)tmp_str, LCD_XOR, LCD_TRANS);
		}

	}

	Draw2D_Rec(0, 18, DISPLAY_W - 1, DISPLAY_H - 19);
	Draw2D_Rec(DISPLAY_W - 10, 18, 9, DISPLAY_H - 19);
	Draw2D_Box(DISPLAY_W - 8, 19 + entry * (DISPLAY_H - 19 - 3) / 10, 6, 3);

	return 0;
}

uint8_t menu_mmc_save (uint8_t cmd) {
	static uint8_t entry = 0;
	uint8_t tmp_str[40];
	int8_t num = 0;
	int8_t x = 0;
	int16_t start_num = 0;

	if (cmd == CMD_ENTER) {
		if (entry > 0) {
			save_setup_mmc(entry - 1);
		}
		redraw = 2;
		return 1;
	} else if (cmd == CMD_UP) {
		if (entry < 10) {
			entry++;
		}
	} else if (cmd == CMD_DOWN) {
		if (entry > 0) {
			entry--;
		}
	}

	Draw2D_Clear();
	DISPLAY_Header((char *)Rc_Model_name);
	Draw2D_PrintXY(0, 10, "Save:", LCD_XOR, LCD_TRANS);

	num = 0;
	start_num = entry - MENU_HEIGHT / 2;
	if (start_num < 0) {
		start_num = 0;
	}
	for (num = 0; num < MENU_HEIGHT && num + start_num < 10; num++) {
		if (num + start_num == entry) {
			Draw2D_Box(2, 20 + (num * 10), DISPLAY_W - 13, 10);
		}

		if (num + start_num == 0) {
			x = Draw2D_PrintXY(3, 22 + (num * 10), (char *)"Back", LCD_XOR, LCD_TRANS);
		} else {
			load_name_mmc((char *)tmp_str, num + start_num - 1);
			x = Draw2D_PrintXY(3, 22 + (num * 10), (char *)tmp_str, LCD_XOR, LCD_TRANS);
		}

	}

	Draw2D_Rec(0, 18, DISPLAY_W - 1, DISPLAY_H - 19);
	Draw2D_Rec(DISPLAY_W - 10, 18, 9, DISPLAY_H - 19);
	Draw2D_Box(DISPLAY_W - 8, 19 + entry * (DISPLAY_H - 19 - 3) / 10, 6, 3);

	return 0;
}


#endif

uint8_t menu_new (uint8_t cmd) {
	char tmp_str[10];
	uint8_t mid = 0;
	uint8_t x = 0;
	int8_t num = 0;
	int16_t start_num = 0;
	static uint8_t entry = 0;
	static uint8_t stage = 0;

	Draw2D_Clear();
	DISPLAY_Header((char *)Rc_Model_name);

	const char *menu_text[] = {"Back", "Stick-Mode", "Model-Type", "Servo-Mode", "Create"};
	const char *menu_text_servotype[] = {"Futaba", "Graupner/JR", "Multiplex"};

	const char *menu_text_modeltype[] = {"Glider-1", "Glider-2", "Glider-3", "Plane-1", "Plane-2", "Acro-1", "Delta-1", "Heli-120", "Heli-180", "Heli-90", "Car", "Boat", "Function"};
	enum {GLIDER_1, GLIDER_2, GLIDER_3, PLANE_1, PLANE_2, ACRO_1, DELTA_1, HELI_120, HELI_180, HELI_90, HELI_CAR, HELI_BOAT, HELI_FUNCTION, MODEL_TYPE_MAX};

	const uint8_t menu_len[] = {4, MODEL_TYPE_MAX, 3};
	static uint8_t menu_value[] = {1, 0, 0, 0, 0, 0, 0, 0};

	x = Draw2D_PrintXY(0, 8, (char *)"Wizard", LCD_XOR, LCD_TRANS) + 8;
	if (stage != 0) {
		x = Draw2D_PrintXY(x, 8, (char *)menu_text[stage], LCD_XOR, LCD_TRANS);
	}

	switch (stage) {
		case 0:
			if (cmd == CMD_ENTER) {
				redraw = 2;
				if (entry == 0) {
					entry = 0;
					return 1;
				} else if (entry == 4) {
					entry = 0;
					Rc_Cmd_Init();
					switch (menu_value[1]) {
						case GLIDER_1:
						case GLIDER_2:
						case GLIDER_3:
						case PLANE_1:
						case PLANE_2:
						case ACRO_1:
						case DELTA_1:
							strcpy((char *)Rc_Model_name, (char *)menu_text_modeltype[menu_value[1]]);
							switch (menu_value[0]) {
								case 0:
									STICK_PLANE_AILERON = 0;
									STICK_PLANE_ELEVATOR = 2;
									STICK_PLANE_THROTTLE = 1;
									STICK_PLANE_RUDDER = 3;
								break;
								case 1:
									STICK_PLANE_AILERON = 0;
									STICK_PLANE_ELEVATOR = 1;
									STICK_PLANE_THROTTLE = 2;
									STICK_PLANE_RUDDER = 3;
								break;
								case 2:
									STICK_PLANE_AILERON = 3;
									STICK_PLANE_ELEVATOR = 1;
									STICK_PLANE_THROTTLE = 2;
									STICK_PLANE_RUDDER = 0;
								break;
								case 3:
									STICK_PLANE_AILERON = 3;
									STICK_PLANE_ELEVATOR = 2;
									STICK_PLANE_THROTTLE = 1;
									STICK_PLANE_RUDDER = 0;
								break;
							}
							strcpy((char *)Rc_Control_name[STICK_PLANE_AILERON], (char *)name_aileron);
							strcpy((char *)Rc_Control_name[STICK_PLANE_ELEVATOR], (char *)name_elevator);
							strcpy((char *)Rc_Control_name[STICK_PLANE_THROTTLE], (char *)name_throttle);
							strcpy((char *)Rc_Control_name[STICK_PLANE_RUDDER], (char *)name_rudder);
							strcpy((char *)Rc_Control_name[CONTROL_PLANE_FLAPS], (char *)name_flaps);
							strcpy((char *)Rc_Control_name[SWITCH_PLANE_EXPO], (char *)"Expo");
							strcpy((char *)Rc_Control_name[SWITCH_PLANE_DUALRATE], (char *)"Dualrate");
							strcpy((char *)Rc_Control_name[SWITCH_PLANE_3D], (char *)"3D");


							switch (menu_value[2]) {
								case 0:
									SERVO_PLANE_AILERON_L = 0;
									SERVO_PLANE_ELEVATOR_L = 1;
									SERVO_PLANE_THROTTLE = 2;
									SERVO_PLANE_RUDDER = 3;
									SERVO_PLANE_AILERON_R = 4;
									SERVO_PLANE_ELEVATOR_R = 5;
								break;
								case 1:
									SERVO_PLANE_AILERON_L = 0;
									SERVO_PLANE_ELEVATOR_L = 1;
									SERVO_PLANE_THROTTLE = 2;
									SERVO_PLANE_RUDDER = 3;
									SERVO_PLANE_AILERON_R = 4;
									SERVO_PLANE_ELEVATOR_R = 5;
								break;
								case 2:
									SERVO_PLANE_AILERON_L = 0;
									SERVO_PLANE_ELEVATOR_L = 1;
									SERVO_PLANE_THROTTLE = 2;
									SERVO_PLANE_RUDDER = 3;
									SERVO_PLANE_AILERON_R = 4;
									SERVO_PLANE_ELEVATOR_R = 5;
								break;
							}
							strcpy((char *)Rc_Output_name[SERVO_PLANE_AILERON_L], (char *)name_aileron1);
							strcpy((char *)Rc_Output_name[SERVO_PLANE_ELEVATOR_L], (char *)name_elevator1);
							strcpy((char *)Rc_Output_name[SERVO_PLANE_THROTTLE], (char *)name_throttle);
							strcpy((char *)Rc_Output_name[SERVO_PLANE_RUDDER], (char *)name_rudder);
							strcpy((char *)Rc_Output_name[SERVO_PLANE_AILERON_R], (char *)name_aileron2);
							strcpy((char *)Rc_Output_name[SERVO_PLANE_ELEVATOR_R], (char *)name_elevator2);
							strcpy((char *)Rc_Output_name[6], (char *)name_flaps);

							// Aileron
							Rc_Mixer_channel_in[mid] = STICK_PLANE_AILERON;
							Rc_Mixer_channel_out1[mid] = SERVO_PLANE_AILERON_L;
							Rc_Mixer_percent_value1[mid] = 100;
							Rc_Mixer_channel_out2[mid] = SERVO_PLANE_AILERON_R;
							Rc_Mixer_percent_value2[mid] = -100;
							mid++;

							// Elevator
							Rc_Mixer_channel_in[mid] = STICK_PLANE_ELEVATOR;
							Rc_Mixer_channel_out1[mid] = SERVO_PLANE_ELEVATOR_L;
							Rc_Mixer_percent_value1[mid] = 100;
							Rc_Mixer_channel_out2[mid] = SERVO_PLANE_ELEVATOR_R;
							Rc_Mixer_percent_value2[mid] = 100;
							mid++;

							// Rudder
							Rc_Mixer_channel_in[mid] = STICK_PLANE_RUDDER;
							Rc_Mixer_channel_out1[mid] = SERVO_PLANE_RUDDER;
							Rc_Mixer_percent_value1[mid] = 100;
							mid++;

							if (menu_value[1] == GLIDER_1 || menu_value[1] == GLIDER_2 || menu_value[1] == GLIDER_3) {
								// Flaps
								Rc_Mixer_channel_in[mid] = CONTROL_PLANE_FLAPS;
								Rc_Mixer_channel_out1[mid] = SERVO_PLANE_FLAPS;
								Rc_Mixer_percent_value1[mid] = 100;
								mid++;
							} else {
								// Throttle
								Rc_Mixer_channel_in[mid] = STICK_PLANE_THROTTLE;
								Rc_Mixer_channel_out1[mid] = SERVO_PLANE_THROTTLE;
								Rc_Mixer_percent_value1[mid] = 100;
								mid++;
							}

							// Flaperon
							Rc_Mixer_channel_in[mid] = CONTROL_PLANE_FLAPS;
							Rc_Mixer_channel_out1[mid] = SERVO_PLANE_AILERON_L;
							Rc_Mixer_percent_value1[mid] = 100;
							Rc_Mixer_channel_out2[mid] = SERVO_PLANE_AILERON_R;
							Rc_Mixer_percent_value2[mid] = 100;
							mid++;

							// Extrem-3D
							if (menu_value[1] == ACRO_1) {
								// Aileron -> Elevator
								Rc_Mixer_switch[mid] = SWITCH_PLANE_3D;
								Rc_Mixer_channel_in[mid] = STICK_PLANE_AILERON;
								Rc_Mixer_channel_out1[mid] = SERVO_PLANE_ELEVATOR_L;
								Rc_Mixer_percent_value1[mid] = 75;
								Rc_Mixer_channel_out2[mid] = SERVO_PLANE_ELEVATOR_R;
								Rc_Mixer_percent_value2[mid] = -75;
								mid++;

								// Elevator -> Aileron
								Rc_Mixer_switch[mid] = SWITCH_PLANE_3D;
								Rc_Mixer_channel_in[mid] = STICK_PLANE_ELEVATOR;
								Rc_Mixer_channel_out1[mid] = SERVO_PLANE_AILERON_L;
								Rc_Mixer_percent_value1[mid] = 75;
								Rc_Mixer_channel_out2[mid] = SERVO_PLANE_AILERON_R;
								Rc_Mixer_percent_value2[mid] = 75;
								mid++;
							}


							// Expo
							Rc_Expo_switch[0] = SWITCH_PLANE_EXPO;
							Rc_Expo_channel[0] = STICK_PLANE_AILERON;
							Rc_Expo_switch[1] = SWITCH_PLANE_EXPO;
							Rc_Expo_channel[1] = STICK_PLANE_ELEVATOR;
							Rc_Expo_switch[2] = SWITCH_PLANE_EXPO;
							Rc_Expo_channel[2] = STICK_PLANE_RUDDER;

							// Dualrate
							Rc_Dualrate_switch[0] = SWITCH_PLANE_DUALRATE;
							Rc_Dualrate_channel[0] = STICK_PLANE_AILERON;
							Rc_Dualrate_switch[1] = SWITCH_PLANE_DUALRATE;
							Rc_Dualrate_channel[1] = STICK_PLANE_ELEVATOR;
							Rc_Dualrate_switch[2] = SWITCH_PLANE_DUALRATE;
							Rc_Dualrate_channel[2] = STICK_PLANE_RUDDER;


						break;
						case HELI_120:
						case HELI_180:
						case HELI_90:
							strcpy((char *)Rc_Model_name, (char *)menu_text_modeltype[menu_value[1]]);
							switch (menu_value[0]) {
								case 0:
									STICK_HELI_ROLL = 0;
									STICK_HELI_NICK = 2;
									STICK_HELI_PITCH = 1;
									STICK_HELI_RUDDER = 3;
								break;
								case 1:
									STICK_HELI_ROLL = 0;
									STICK_HELI_NICK = 1;
									STICK_HELI_PITCH = 2;
									STICK_HELI_RUDDER = 3;
								break;
								case 2:
									STICK_HELI_ROLL = 3;
									STICK_HELI_NICK = 1;
									STICK_HELI_PITCH = 2;
									STICK_HELI_RUDDER = 0;
								break;
								case 3:
									STICK_HELI_ROLL = 3;
									STICK_HELI_NICK = 2;
									STICK_HELI_PITCH = 1;
									STICK_HELI_RUDDER = 0;
								break;
							}
							strcpy((char *)Rc_Control_name[STICK_HELI_ROLL], (char *)name_roll);
							strcpy((char *)Rc_Control_name[STICK_HELI_NICK], (char *)name_nick);
							strcpy((char *)Rc_Control_name[STICK_HELI_PITCH], (char *)name_pitch);
							strcpy((char *)Rc_Control_name[STICK_HELI_RUDDER], (char *)name_rudder);
							strcpy((char *)Rc_Control_name[SWITCH_HELI_EXPO], (char *)"Expo");
							strcpy((char *)Rc_Control_name[SWITCH_HELI_DUALRATE], (char *)"Dualrate");
							strcpy((char *)Rc_Control_name[SWITCH_HELI_3D], (char *)"3D");
							strcpy((char *)Rc_Control_name[SWITCH_HELI_ENGINE], (char *)"Engine");
							strcpy((char *)Rc_Control_name[SWITCH_HELI_HOLD], (char *)"Hold");

							switch (menu_value[2]) {
								case 0:
									SERVO_HELI_ROLL_L = 0;
									SERVO_HELI_ROLL_R = 1;
									SERVO_HELI_NICK = 2;
									SERVO_HELI_RUDDER = 3;
									SERVO_HELI_THROTTLE = 4;
								break;
								case 1:
									SERVO_HELI_ROLL_L = 0;
									SERVO_HELI_ROLL_R = 1;
									SERVO_HELI_NICK = 2;
									SERVO_HELI_RUDDER = 3;
									SERVO_HELI_THROTTLE = 4;
								break;
								case 2:
									SERVO_HELI_ROLL_L = 0;
									SERVO_HELI_ROLL_R = 1;
									SERVO_HELI_NICK = 2;
									SERVO_HELI_RUDDER = 3;
									SERVO_HELI_THROTTLE = 4;
								break;
							}
							strcpy((char *)Rc_Output_name[SERVO_HELI_ROLL_L], (char *)name_roll1);
							strcpy((char *)Rc_Output_name[SERVO_HELI_ROLL_R], (char *)name_roll2);
							strcpy((char *)Rc_Output_name[SERVO_HELI_NICK], (char *)name_nick);
							strcpy((char *)Rc_Output_name[SERVO_HELI_RUDDER], (char *)name_rudder);
							strcpy((char *)Rc_Output_name[SERVO_HELI_THROTTLE], (char *)name_throttle);

							if (menu_value[1] == HELI_120) {
								// Roll
								Rc_Mixer_channel_in[mid] = STICK_HELI_ROLL;
								Rc_Mixer_channel_out1[mid] = SERVO_HELI_ROLL_L;
								Rc_Mixer_percent_value1[mid] = 100;
								Rc_Mixer_channel_out2[mid] = SERVO_HELI_ROLL_R;
								Rc_Mixer_percent_value2[mid] = -100;
								mid++;

								// Nick
								Rc_Mixer_channel_in[mid] = STICK_HELI_NICK;
								Rc_Mixer_channel_out1[mid] = SERVO_HELI_ROLL_L;
								Rc_Mixer_percent_value1[mid] = -75;
								Rc_Mixer_channel_out2[mid] = SERVO_HELI_ROLL_R;
								Rc_Mixer_percent_value2[mid] = -75;
								Rc_Mixer_channel_out3[mid] = SERVO_HELI_NICK;
								Rc_Mixer_percent_value3[mid] = 100;
								mid++;

								// Pitch
								Rc_Mixer_channel_in[mid] = STICK_HELI_PITCH;
								Rc_Mixer_channel_out1[mid] = SERVO_HELI_ROLL_L;
								Rc_Mixer_percent_value1[mid] = 100;
								Rc_Mixer_channel_out2[mid] = SERVO_HELI_ROLL_R;
								Rc_Mixer_percent_value2[mid] = 100;
								Rc_Mixer_channel_out3[mid] = SERVO_HELI_NICK;
								Rc_Mixer_percent_value3[mid] = 100;
								Rc_Mixer_channel_out4[mid] = SERVO_HELI_RUDDER;
								Rc_Mixer_percent_value4[mid] = -10;
								mid++;
							} else {
								strcpy((char *)Rc_Output_name[SERVO_HELI_ROLL_L], (char *)name_roll);
								strcpy((char *)Rc_Output_name[SERVO_HELI_ROLL_R], (char *)name_pitch);

								// Roll
								Rc_Mixer_channel_in[mid] = STICK_HELI_ROLL;
								Rc_Mixer_channel_out1[mid] = SERVO_HELI_ROLL;
								Rc_Mixer_percent_value1[mid] = 100;
								mid++;

								// Nick
								Rc_Mixer_channel_in[mid] = STICK_HELI_NICK;
								Rc_Mixer_channel_out1[mid] = SERVO_HELI_NICK;
								Rc_Mixer_percent_value1[mid] = 100;
								mid++;

								// Pitch
								Rc_Mixer_channel_in[mid] = STICK_HELI_PITCH;
								Rc_Mixer_channel_out1[mid] = SERVO_HELI_PITCH;
								Rc_Mixer_percent_value1[mid] = 100;
								Rc_Mixer_channel_out4[mid] = SERVO_HELI_RUDDER;
								Rc_Mixer_percent_value4[mid] = -10;
								mid++;
							}

							// Throttle/Pitch
							Rc_Mixer_channel_in[mid] = STICK_HELI_PITCH;
							Rc_Mixer_channel_out1[mid] = SERVO_HELI_THROTTLE;
							Rc_Mixer_percent_value1[mid] = 100;
							Rc_Mixer_channel_out2[mid] = SERVO_HELI_RUDDER;
							Rc_Mixer_percent_value2[mid] = -10;
							Rc_Mixer_point[mid][0][1] = 10;
							Rc_Mixer_point[mid][1][1] = 20;
							Rc_Mixer_point[mid][2][1] = 40;
							Rc_Mixer_point[mid][3][1] = 75;
							Rc_Mixer_point[mid][4][1] = 85;
							Rc_Mixer_point[mid][5][1] = 95;
							Rc_Mixer_point[mid][6][1] = 100;
							mid++;

							// Rudder
							Rc_Mixer_channel_in[mid] = STICK_HELI_RUDDER;
							Rc_Mixer_channel_out1[mid] = SERVO_HELI_RUDDER;
							Rc_Mixer_percent_value1[mid] = 100;
							mid++;

							// Limiter / Engine-Control
							Rc_Limiter_switch[0] = SWITCH_HELI_HOLD;
							Rc_Limiter_channel[0] = SERVO_HELI_THROTTLE;
							Rc_Limiter_up_value[0] = 100;
							Rc_Limiter_down_value[0] = 60;

							Rc_Limiter_switch[1] = SWITCH_HELI_ENGINE;
							Rc_Limiter_channel[1] = SERVO_HELI_THROTTLE;
							Rc_Limiter_up_value[1] = 0;
							Rc_Limiter_down_value[1] = 0;

							// Expo
							Rc_Expo_switch[0] = SWITCH_HELI_EXPO;
							Rc_Expo_channel[0] = STICK_HELI_ROLL;
							Rc_Expo_switch[1] = SWITCH_HELI_EXPO;
							Rc_Expo_channel[1] = STICK_HELI_NICK;
							Rc_Expo_switch[2] = SWITCH_HELI_EXPO;
							Rc_Expo_channel[2] = STICK_HELI_RUDDER;

							// Dualrate
							Rc_Dualrate_switch[0] = SWITCH_HELI_DUALRATE;
							Rc_Dualrate_channel[0] = STICK_HELI_ROLL;
							Rc_Dualrate_switch[1] = SWITCH_HELI_DUALRATE;
							Rc_Dualrate_channel[1] = STICK_HELI_NICK;
							Rc_Dualrate_switch[2] = SWITCH_HELI_DUALRATE;
							Rc_Dualrate_channel[2] = STICK_HELI_RUDDER;
						break;
						default:
							// HELI_CAR, HELI_BOAT, HELI_FUNCTION
							strcpy((char *)Rc_Model_name, "Std.");
							strcpy((char *)Rc_Control_name[0], (char *)"Lenkung");
							strcpy((char *)Rc_Control_name[1], (char *)"-");
							strcpy((char *)Rc_Control_name[2], (char *)"Gas");
							strcpy((char *)Rc_Control_name[3], (char *)"Getriebe");
						break;
					}

					return 1;
				} else {
					cmd = 0;
					stage = entry;
				}
			} else if (cmd == CMD_UP) {
				if (entry < 4) {
					entry++;
				} else {
					entry = 0;
				}
			} else if (cmd == CMD_DOWN) {
				if (entry > 0) {
					entry--;
				} else {
					entry = 4;
				}
			}
			Draw2D_Font_SetNum(1);
		break;
		default:
			if (cmd == CMD_ENTER) {
				redraw = 2;
				stage = 0;
			} else if (cmd == CMD_UP) {
				if (menu_value[stage - 1] < menu_len[stage - 1] - 1) {
					menu_value[stage - 1]++;
				} else {
					menu_value[stage - 1] = 0;
				}
			} else if (cmd == CMD_DOWN) {
				if (menu_value[stage - 1] > 0) {
					menu_value[stage - 1]--;
				} else {
					menu_value[stage - 1] = menu_len[stage - 1] - 1;
				}
			}
		break;
	}

	num = 0;
	start_num = entry - MENU_HEIGHT / 2;
	if (start_num < 0) {
		start_num = 0;
	}
	for (num = 0; num < MENU_HEIGHT && num + start_num < 5; num++) {
		if (num + start_num == entry) {
			Draw2D_Box(2, 20 + (num * 10), DISPLAY_W - 13, 10);
		}
		x = Draw2D_PrintXY(3, 22 + (num * 10), (char *)menu_text[num + start_num], LCD_XOR, LCD_TRANS);
		switch (num + start_num) {
			case 1:
				x = Draw2D_PrintXY(x + 2, 22 + (num * 10), (char *)":", LCD_XOR, LCD_TRANS);
				int2str_new(tmp_str, menu_value[num + start_num - 1] + 1, 0, 0, 1);
				Draw2D_PrintXY(x + 3, 22 + (num * 10), (char *)tmp_str, LCD_XOR, LCD_TRANS);
			break;
			case 2:
				x = Draw2D_PrintXY(x + 2, 22 + (num * 10), (char *)":", LCD_XOR, LCD_TRANS);
				Draw2D_PrintXY(x + 3, 22 + (num * 10), (char *)menu_text_modeltype[menu_value[num + start_num - 1]], LCD_XOR, LCD_TRANS);
			break;
			case 3:
				x = Draw2D_PrintXY(x + 2, 22 + (num * 10), (char *)":", LCD_XOR, LCD_TRANS);
				Draw2D_PrintXY(x + 3, 22 + (num * 10), (char *)menu_text_servotype[menu_value[num + start_num - 1]], LCD_XOR, LCD_TRANS);
			break;
		}
	}
	Draw2D_Font_SetNum(1);

	return 0;
}

uint8_t menu_volt_temp (uint8_t cmd) {
	uint8_t tmp_str[40];

	if (cmd == CMD_ENTER) {
		redraw = 2;
		return 1;
	}

	Draw2D_Clear();
	DISPLAY_Header((char *)Rc_Model_name);

	uint16_t temperature = (int16_t)(((1.4 - (float)GetADCChanel(ADC_CHANNELS_MAX - 1) * 3.3 / 4096.0) / 0.004478) + 25.0);
	uint16_t voltage = (int16_t)((float)GetADCChanel(ADC_CHANNELS_MAX - 2) * 15.6 / 4096.0 * 100.0);
	tmp_str[0] = (temperature / 100) + 0x30;
	tmp_str[1] = ((temperature % 100) / 10) + 0x30;
	tmp_str[2] = '.';
	tmp_str[3] = ((temperature % 100) % 10) + 0x30;
	tmp_str[4] = 'C';
	tmp_str[5] = 0;
	Draw2D_PrintXY(0, 8, "CPU-Temp.  ", LCD_XOR, LCD_TRANS);
	Draw2D_Font_SetNum(2);
	Draw2D_PrintXY(255, 16, (char *)tmp_str, LCD_XOR, LCD_TRANS);
	Draw2D_Font_SetNum(1);
	tmp_str[0] = ' ';
	if (voltage > 1000) {
		tmp_str[1] = (voltage / 1000) + 0x30;
	} else {
		tmp_str[1] = ' ';
	}
	tmp_str[2] = ((voltage % 1000) / 100) + 0x30;
	tmp_str[3] = '.';
	tmp_str[4] = (((voltage % 1000) % 100) / 10) + 0x30;
	tmp_str[5] = (((voltage % 1000) % 100) % 10) + 0x30;
	tmp_str[6] = 'V';
	tmp_str[7] = 0;
	Draw2D_PrintXY(0, 24, "Battery   ", LCD_XOR, LCD_TRANS);
	Draw2D_Font_SetNum(2);
	Draw2D_PrintXY(255, 32, (char *)tmp_str, LCD_XOR, LCD_TRANS);
	Draw2D_Font_SetNum(1);

	return 0;
}

uint8_t menu_edit_uint8 (uint8_t cmd, uint8_t *value) {
	uint8_t tmp_str[40];
	if (cmd == CMD_ENTER) {
		redraw = 2;
		return 1;
	} else if (cmd == CMD_UP) {
		if (value[0] < 255) {
			value[0]++;
		} else {
			value[0] = 0;
		}
	} else if (cmd == CMD_DOWN) {
		if (value[0] > 0) {
			value[0]--;
		} else {
			value[0] = 255;
		}
	}
	Draw2D_PrintXY(0, 24, (char *)"Value (uint8)", LCD_XOR, LCD_TRANS);

	int2str_new((char *)tmp_str, (int16_t)(value[0]), 0, 0, 1);
	Draw2D_PrintXY(255, 32, (char *)tmp_str, LCD_XOR, LCD_TRANS);
	Draw2D_PrintXY(255, 48, (char *)"%", LCD_XOR, LCD_TRANS);

	return 0;
}

uint8_t menu_edit_int8 (int8_t cmd, int8_t *value) {
	uint8_t tmp_str[40];
	if (cmd == CMD_ENTER) {
		redraw = 2;
		return 1;
	} else if (cmd == CMD_UP) {
		if (value[0] < 127) {
			value[0]++;
		} else {
			value[0] = 0;
		}
	} else if (cmd == CMD_DOWN) {
		if (value[0] > -127) {
			value[0]--;
		} else {
			value[0] = 127;
		}
	}
	Draw2D_PrintXY(0, 24, (char *)"Value (int8)", LCD_XOR, LCD_TRANS);

	int2str_new((char *)tmp_str, (int16_t)(value[0]), 0, 0, 1);
	Draw2D_PrintXY(255, 32, (char *)tmp_str, LCD_XOR, LCD_TRANS);
	Draw2D_PrintXY(255, 48, (char *)"%", LCD_XOR, LCD_TRANS);

	return 0;
}

uint8_t menu_edit_int16 (int8_t cmd, int16_t *value) {
	uint8_t tmp_str[40];
	if (cmd == CMD_ENTER) {
		redraw = 2;
		return 1;
	} else if (cmd == CMD_UP) {
		if (value[0] < 0xFFFF / 2) {
			value[0]++;
		} else {
			value[0] = 0;
		}
	} else if (cmd == CMD_DOWN) {
		if (value[0] > -32000) {
			value[0]--;
		} else {
			value[0] = 0xFFFF / 2;
		}
	}
	Draw2D_PrintXY(0, 24, (char *)"Value (int16)", LCD_XOR, LCD_TRANS);

	int2str_new((char *)tmp_str, (int16_t)(value[0]), 0, 0, 1);
	Draw2D_PrintXY(255, 32, (char *)tmp_str, LCD_XOR, LCD_TRANS);

	return 0;
}

uint8_t menu_edit_uint16 (int8_t cmd, uint16_t *value) {
	uint8_t tmp_str[40];
	if (cmd == CMD_ENTER) {
		redraw = 2;
		return 1;
	} else if (cmd == CMD_UP) {
		if (value[0] < 0xFFFF) {
			value[0]++;
		} else {
			value[0] = 0;
		}
	} else if (cmd == CMD_DOWN) {
		if (value[0] > 0) {
			value[0]--;
		} else {
			value[0] = 0xFFFF;
		}
	}
	Draw2D_PrintXY(0, 24, (char *)"Value (uint16)", LCD_XOR, LCD_TRANS);
	int2str_new((char *)tmp_str, (int16_t)(value[0]), 0, 0, 1);
	Draw2D_PrintXY(0, 32, (char *)tmp_str, LCD_XOR, LCD_TRANS);
	return 0;
}

uint8_t menu_edit_text (int8_t cmd, char *text) {
	static uint8_t entry = 0;
	uint8_t tmp_str[40];
	if (cmd == CMD_ENTER) {
		if (text[entry] == 31) {
			entry = 0;
		} else if (text[entry] == 30) {
			text[entry] = 0;
			entry = 0;
			redraw = 2;
			return 1;
		} else {
			entry++;
		}
		redraw = 2;
	} else if (cmd == CMD_UP) {
		if (text[entry] < 124) {
			text[entry]++;
		} else {
			text[entry] = 124;
		}
	} else if (cmd == CMD_DOWN) {
		if (text[entry] > 30) {
			text[entry]--;
		} else {
			text[entry] = 30;
		}
	}
	strcpy((char *)tmp_str, text);
	tmp_str[entry] = 0;
	uint8_t next_x = Draw2D_PrintXY(10, 32, (char *)tmp_str, LCD_XOR, LCD_TRANS);
	Draw2D_Font_SetNum(2);
	strcpy((char *)tmp_str, text + entry);
	tmp_str[1] = 0;
	next_x = Draw2D_PrintXY(next_x + 5, 32, (char *)tmp_str, LCD_XOR, LCD_TRANS);
	Draw2D_Font_SetNum(1);
	strcpy((char *)tmp_str, text + entry + 1);
	next_x = Draw2D_PrintXY(next_x + 5, 32, (char *)tmp_str, LCD_XOR, LCD_TRANS);
	Draw2D_Font_SetNum(2);
	if (text[entry] == 30) {
		Draw2D_StringCenter(48, (char *)name_ok, LCD_XOR, LCD_TRANS);
	} else if (text[entry] == 31) {
		Draw2D_StringCenter(48, (char *)name_back, LCD_XOR, LCD_TRANS);
	}
	Draw2D_Font_SetNum(1);
	return 0;
}


uint8_t menu_edit_alarm (int8_t cmd, uint32_t *timer_alarm) {
	static uint8_t entry = 0;
	uint8_t tmp_str[20];
	uint8_t pos_x = 10;
	if (cmd == CMD_ENTER) {
		redraw = 2;
		if (entry == 0) {
			entry++;
		} else if (entry == 1) {
			entry = 0;
			return 1;
		}
	} else if (cmd == CMD_UP) {
		if (entry == 0) {
			timer_alarm[0] += 60000;
		} else {
			timer_alarm[0] += 1000;
		}
	} else if (cmd == CMD_DOWN) {
		if (entry == 0) {
			timer_alarm[0] -= 60000;
		} else {
			timer_alarm[0] -= 1000;
		}
	}

	if (entry == 0) {
		Draw2D_Font_SetNum(2);
	} else {
		Draw2D_Font_SetNum(1);
	}
	int2str_new((char *)tmp_str, (timer_alarm[0] / 1000) / 60, 1, 0, 1);
	pos_x = Draw2D_PrintXY(pos_x, 48, (char *)tmp_str, LCD_XOR, LCD_TRANS);
	Draw2D_Font_SetNum(1);
	pos_x = Draw2D_PrintXY(pos_x, 48, (char *)":", LCD_XOR, LCD_TRANS);
	if (entry == 1) {
		Draw2D_Font_SetNum(2);
	}
	int2str_new((char *)tmp_str, (uint16_t)((timer_alarm[0] / 1000) - ((timer_alarm[0] / 1000) / 60 * 60)), 1, 0, 1);
	pos_x = Draw2D_PrintXY(pos_x, 48, (char *)tmp_str, LCD_XOR, LCD_TRANS);

	Draw2D_Font_SetNum(1);

	return 0;
}


uint8_t menu_edit_points (uint8_t cmd, uint8_t mixer_id) {
	static uint8_t entry = 0;
	static uint8_t stage = 0;
	if (stage == 0) {
		if (cmd == CMD_ENTER) {
			redraw = 2;
			if (entry == 0) {
				return 1;
			} else {
				cmd = 0;
				stage = entry;
			}
		} else if (cmd == CMD_UP) {
			if (entry < (MIXER_POINTS_MAX * 2) - 1) {
				entry++;
			}
		} else if (cmd == CMD_DOWN) {
			if (entry > 0) {
				entry--;
			}
		}
	} else {
		if (cmd == CMD_ENTER) {
			redraw = 2;
			stage = 0;
		} else if (cmd == CMD_UP) {
			if (Rc_Mixer_point[mixer_id][entry / 2][entry - (entry / 2 * 2)] < 100) {
				Rc_Mixer_point[mixer_id][entry / 2][entry - (entry / 2 * 2)]++;
			}
		} else if (cmd == CMD_DOWN) {
			if (Rc_Mixer_point[mixer_id][entry / 2][entry - (entry / 2 * 2)] > 0) {
				Rc_Mixer_point[mixer_id][entry / 2][entry - (entry / 2 * 2)]--;
			}
		}
	}

	if (entry == 0) {
		Draw2D_Font_SetNum(2);
		Draw2D_StringCenter(24, (char *)name_back, LCD_XOR, LCD_TRANS);
	} else {
		Draw2D_Font_SetNum(0);
		print_mixer(0, (DISPLAY_H / 3), (DISPLAY_W - 2), (DISPLAY_H * 2 / 3), mixer_id, (entry) / 2, entry - (entry / 2 * 2));
	}
	Draw2D_Font_SetNum(1);

	return 0;
}

uint8_t menu_edit_control (uint8_t cmd, uint8_t *value) {
	if (cmd == CMD_ENTER) {
		redraw = 2;
		return 1;
	} else if (cmd == CMD_UP) {
		if (value[0] < CONTROL_MAX) {
			value[0]++;
		} else {
			value[0] = 0;
		}
	} else if (cmd == CMD_DOWN) {
		if (value[0] > 0) {
			value[0]--;
		} else {
			value[0] = CONTROL_MAX;
		}
	}

	if (Last_Control != 255) {
		value[0] = Last_Control;
		Last_Control = 255;
	}

	Draw2D_PrintXY(0, 24, (char *)"Control", LCD_XOR, LCD_TRANS);
	print_control(16, 32, value[0], LCD_XOR, LCD_TRANS);
	return 0;
}

uint8_t menu_edit_output (uint8_t cmd, uint8_t *value) {
	if (cmd == CMD_ENTER) {
		redraw = 2;
		return 1;
	} else if (cmd == CMD_UP) {
		if (value[0] < OUTPUT_MAX) {
			value[0]++;
		} else {
			value[0] = 0;
		}
	} else if (cmd == CMD_DOWN) {
		if (value[0] > 0) {
			value[0]--;
		} else {
			value[0] = OUTPUT_MAX;
		}
	}
	Draw2D_PrintXY(0, 24, (char *)"Control", LCD_XOR, LCD_TRANS);
	print_output(10, 32, value[0], LCD_XOR, LCD_TRANS);

	return 0;
}

uint8_t menu_edit_flag (int8_t cmd, int8_t *value) {
	value[0] = 1 - value[0];
	redraw = 2;
	return 1;
}

uint8_t menu_system (uint8_t cmd) {
	static uint8_t entry = 0;
	static uint8_t stage = 0;
	uint8_t x = 0;
	uint8_t menu_len = 11;
	const char *text[] = {name_back, "Info", "New", "Load", "Save", "Control-Monitor", "Servo-Monitor", "ADC-Monitor", "Calibrate", "Control-Popup", "FMS-Output", "Reload-MMC", "---", "---"};

	if (cmd == CMD_ENTER) {
		redraw = 2;
		if (stage == 0) {
			if (entry == 0) {
				return 1;
			} else {
				cmd = 0;
				stage = entry;
			}
		}
	} else if (cmd == CMD_UP && entry < menu_len) {
		entry++;
	} else if (cmd == CMD_DOWN && entry > 0) {
		entry--;
	}

	if (stage == 0) {
		Draw2D_Clear();
		DISPLAY_Header((char *)Rc_Model_name);
		Draw2D_PrintXY(0, 10, (char *)"System-Menu", LCD_XOR, LCD_TRANS);
		int8_t num = 0;
		int16_t start_num = entry - MENU_HEIGHT / 2;
		if (start_num < 0) {
			start_num = 0;
		}
		for (num = 0; num < MENU_HEIGHT && num + start_num < menu_len + 1; num++) {
			if (num + start_num == entry) {
				Draw2D_Box(2, 20 + (num * 10), DISPLAY_W - 13, 10);
			}
			x = Draw2D_PrintXY(10, 22 + (num * 10), (char *)text[num + start_num], LCD_XOR, LCD_TRANS);
			if (num + start_num == 9) {
				if (control_change_popup == 1) {
					x = Draw2D_PrintXY(x + 5, 22 + (num * 10), (char *)"On", LCD_XOR, LCD_TRANS);
				} else {
					x = Draw2D_PrintXY(x + 5, 22 + (num * 10), (char *)"Off", LCD_XOR, LCD_TRANS);
				}
			}
			if (num + start_num == 10) {
				if (fms_output == 1) {
					x = Draw2D_PrintXY(x + 5, 22 + (num * 10), (char *)"On", LCD_XOR, LCD_TRANS);
				} else {
					x = Draw2D_PrintXY(x + 5, 22 + (num * 10), (char *)"Off", LCD_XOR, LCD_TRANS);
				}
			}
		}
		Draw2D_Rec(0, 18, DISPLAY_W - 1, DISPLAY_H - 19);
		Draw2D_Rec(DISPLAY_W - 10, 18, 9, DISPLAY_H - 19);
		Draw2D_Box(DISPLAY_W - 8, 19 + entry * (DISPLAY_H - 19 - 3) / 5, 6, 3);
	} else if (stage == 1) {
		if (menu_volt_temp(cmd) == 1) {
			stage = 0;
		}
	} else if (stage == 2) {
		if (menu_new(cmd) == 1) {
			stage = 0;
		}
#ifdef SUPPORT_MMC
	} else if (stage == 3) {
		if (menu_mmc_load(cmd) == 1) {
			stage = 0;
		}
	} else if (stage == 4) {
		if (menu_mmc_save(cmd) == 1) {
			stage = 0;
		}
#endif

	} else if (stage == 5) {
		if (menu_control_overview(cmd) == 1) {
			stage = 0;
		}
	} else if (stage == 6) {
		if (menu_output_overview(cmd) == 1) {
			stage = 0;
		}
	} else if (stage == 7) {
		if (menu_adc(cmd) == 1) {
			stage = 0;
		}
	} else if (stage == 8) {
		if (menu_calibrate(cmd) == 1) {
			stage = 0;
		}
	} else if (stage == 9) {
		control_change_popup = 1 - control_change_popup;
		stage = 0;
	} else if (stage == 10) {
		fms_output = 1 - fms_output;
		stage = 0;
	} else if (stage == 11) {
#ifdef SUPPORT_MMC
		rc_mmc_init();
#endif
		stage = 0;
	}
	return 0;
}

void menu_list_view_id (uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t id, uint8_t selected) {
	uint8_t text[20];
	strcpy((char *)text, "ID:");
	text[3] = ' ';
	text[4] = '0' + id;
	text[5] = 0;
	if (selected == 1) {
		Draw2D_Box(x, y, w, h);
	}
	Draw2D_PrintXY(x + 4, y + 2, (char *)text, LCD_XOR, LCD_TRANS);
}

void menu_list_view_Timer (uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t id, uint8_t selected) {
	uint8_t tmp_str[20];
	if (selected == 1) {
		Draw2D_Box(x, y, w, h);
	}
	Draw2D_Font_SetNum(1);
	x = Draw2D_PrintXY(x + 4, y + 2, (char *)SystemTimer_name[id], LCD_XOR, LCD_TRANS);
	x = Draw2D_PrintXY(x, y + 2, (char *)":", LCD_XOR, LCD_TRANS);
	int2str_new((char *)tmp_str, (SystemTimer_time[id] / 1000) / 60, 1, 0, 1);
	x = Draw2D_PrintXY(x + 10, y + 2, (char *)tmp_str, LCD_XOR, LCD_TRANS);
	x = Draw2D_PrintXY(x, y + 2, (char *)":", LCD_XOR, LCD_TRANS);
	int2str_new((char *)tmp_str, (uint16_t)((SystemTimer_time[id] / 1000) - ((SystemTimer_time[id] / 1000) / 60 * 60)), 1, 0, 1);
	x = Draw2D_PrintXY(x, y + 2, (char *)tmp_str, LCD_XOR, LCD_TRANS);
}

void menu_list_view_Control (uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t id, uint8_t selected) {
	if (selected == 1) {
		Draw2D_Box(x, y, w, h);
	}
	print_control(x + 4, y + 2, id, LCD_XOR, LCD_TRANS);
}

void menu_list_view_Dualrate (uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t id, uint8_t selected) {
	menu_list_view_Control(x, y, w, h, Rc_Dualrate_channel[id], selected);
}

void menu_list_view_Expo (uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t id, uint8_t selected) {
	menu_list_view_Control(x, y, w, h, Rc_Expo_channel[id], selected);
}

void menu_list_view_Atv (uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t id, uint8_t selected) {
	menu_list_view_Output(x, y, w, h, Rc_Atv_channel[id], selected);
}

void menu_list_view_Limiter (uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t id, uint8_t selected) {
	menu_list_view_Output(x, y, w, h, Rc_Limiter_channel[id], selected);
}

void menu_list_view_Mixer (uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t id, uint8_t selected) {
	if (selected == 1) {
		Draw2D_Box(x, y, w, h);
	}
	Draw2D_Font_SetNum(0);
	x = print_control(x + 2, y + 3, Rc_Mixer_channel_in[id], LCD_XOR, LCD_TRANS);
	x = Draw2D_PrintXY(x + 2, y + 3, (char *)">", LCD_XOR, LCD_TRANS);
	x = print_output(x + 2, y + 3, Rc_Mixer_channel_out1[id], LCD_XOR, LCD_TRANS);
	Draw2D_Font_SetNum(1);
}

void menu_list_view_Output (uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t id, uint8_t selected) {
	if (selected == 1) {
		Draw2D_Box(x, y, w, h);
	}
	print_output(x + 4, y + 2, id, LCD_XOR, LCD_TRANS);
}

uint8_t menu_select (char *title, uint8_t cmd, uint8_t len, void (*list_view) (uint8_t x, uint8_t y, uint8_t w, uint8_t h, uint8_t id, uint8_t selected), uint8_t (*view)(uint8_t cmd, uint8_t id)) {
	static uint8_t entry = 0;
	static uint8_t stage = 0;
	if (stage == 0) {
		if (cmd == CMD_ENTER) {
			redraw = 2;
			if (stage == 0) {
				if (entry == 0) {
					return 1;
				} else {
					cmd = 0;
					stage = entry;
					entry = 0;
				}
			}
		} else if (cmd == CMD_UP && entry < len + 1) {
			entry++;
		} else if (cmd == CMD_DOWN && entry > 0) {
			entry--;
		}

		Draw2D_Clear();
		DISPLAY_Header((char *)Rc_Model_name);
		Draw2D_PrintXY(0, 10, (char *)title, LCD_XOR, LCD_TRANS);
		int8_t num = 0;
		int16_t start_num = entry - MENU_HEIGHT / 2;
		if (start_num < 0) {
			start_num = 0;
		}
		for (num = 0; num < MENU_HEIGHT && num + start_num < len + 1; num++) {
			if (num + start_num == 0) {
				if (num + start_num == entry) {
					Draw2D_Box(2, 20 + (num * 10), DISPLAY_W - 13, 10);
				}
				Draw2D_PrintXY(2, 20 + (num * 10), (char *)name_back, LCD_XOR, LCD_TRANS);
			} else {
				if (num + start_num == entry) {
					(*list_view)(2, 20 + (num * 10), DISPLAY_W - 13, 10, start_num + num - 1, 1);
				} else {
					(*list_view)(2, 20 + (num * 10), DISPLAY_W - 13, 10, start_num + num - 1, 0);
				}
			}
		}
		Draw2D_Rec(0, 18, DISPLAY_W - 1, DISPLAY_H - 19);
		Draw2D_Rec(DISPLAY_W - 10, 18, 9, DISPLAY_H - 19);
		Draw2D_Box(DISPLAY_W - 8, 19 + entry * (DISPLAY_H - 19 - 3) / 5, 6, 3);
	} else {
		Draw2D_Clear();
		DISPLAY_Header((char *)Rc_Model_name);
		Draw2D_PrintXY(0, 10, (char *)title, LCD_XOR, LCD_TRANS);
		if ((*view)(cmd, stage - 1) == 1) {
			entry = stage;
			stage = 0;
		}
	}
	return 0;
}

uint8_t menu_find_next_mixer (void) {
	uint8_t num;
	for (num = 0; num < MIXER_MAX; num++) {
		if (Rc_Mixer_channel_in[num] == 255 || Rc_Mixer_channel_out1[num] == 255) {
			return 0;
		}
	}
	return 255;
}

uint8_t menu_find_next_expo (void) {
	uint8_t num;
	for (num = 0; num < EXPO_MAX; num++) {
		if (Rc_Expo_channel[num] == 255) {
			return 0;
		}
	}
	return 255;
}

uint8_t menu_find_next_dualrate (void) {
	uint8_t num;
	for (num = 0; num < DUALRATE_MAX; num++) {
		if (Rc_Dualrate_channel[num] == 255) {
			return 0;
		}
	}
	return 255;
}

uint8_t menu_find_next_atv (void) {
	uint8_t num;
	for (num = 0; num < ATV_MAX; num++) {
		if (Rc_Atv_channel[num] == 255) {
			return 0;
		}
	}
	return 255;
}

uint8_t menu_find_next_motion (void) {
	uint8_t num;
	for (num = 0; num < MOTION_MAX; num++) {
		if (Rc_Motion_channel[num] == 255) {
			return 0;
		}
	}
	return 255;
}

uint8_t menu_find_next_limiter (void) {
	uint8_t num;
	for (num = 0; num < LIMITER_MAX; num++) {
		if (Rc_Limiter_channel[num] == 255) {
			return 0;
		}
	}
	return 255;
}


#endif


